'''
Polychronization: Computation with Spikes
Eugene M. Izhikevich
Neural Computation, 2006

This script shows a way to reproduce Izhikevich's network using Brian. Note
that there are a couple of slightly tricky technical points necessary to
reproduce Izhikevich's particular numerical integration scheme and STDP rule.
These are explained below.

This version of the script demonstrates the simplest version of the code
possible, the longer version of this script shows a much faster version which
saves the output and analyses the polychronous groups formed.
'''

# This is to use interactive plotting
import matplotlib
matplotlib.use('WXAgg') # You may need to experiment, try WXAgg, GTKAgg, QTAgg, TkAgg

from brian import *
import random

dt = defaultclock.dt = .5 * ms

I_factor = float((1*ms)/dt)

M = 100                # number of synapses per neuron
D = 20 * ms              # maximal conduction delay
D_min = 1 * ms           # minimal conduction delay
D_i = 1 * ms             # inhibitory neuron conduction delay

# excitatory neurons
Ne = 800
a_e = 0.02 / ms
d_e = 8 * mV / ms
s_e = 6 * mV / ms * I_factor
# inhibitory neurons
Ni = 200
a_i = 0.1 / ms
d_i = 2 * mV / ms
s_i = -5 * mV / ms * I_factor
# all neurons
b = 0.2 / ms
c = -65 * mV
sm = 10 * mV / ms * I_factor           # maximal synaptic strength
N = Ne + Ni

thalamic_input_weight = 20 * mV / ms# * I_factor

eqs = Equations('''
dv/dt = (0.04/ms/mV)*v**2+(5/ms)*v+140*mV/ms-u+I : volt
du/dt = a*(b*v-u)                                : volt/second
a                                                : 1/second
d                                                : volt/second
I                                                : volt/second
''')
reset = '''
v = c
u += d
'''
threshold = 30 * mV

G = NeuronGroup(N, eqs, threshold=threshold, reset=reset)

# Izhikevich's numerical integration scheme is not built in to Brian, but
# we can specify it explicitly. By setting the NeuronGroup._state_updater
# attribute to a function f(G), we replace the default numerical integration
# scheme (Euler) defined by Brian. This function can run arbitrary Python
# code.
def Izhikevich_scheme(G):
    G.v += 0.5 * dt * ((0.04 / ms / mV) * G.v ** 2 + (5 / ms) * G.v + 140 * mV / ms - G.u + G.I)
    G.v += 0.5 * dt * ((0.04 / ms / mV) * G.v ** 2 + (5 / ms) * G.v + 140 * mV / ms - G.u + G.I)
    G.u += dt * G.a * (b * G.v - G.u)
G._state_updater = Izhikevich_scheme

Ge = G.subgroup(Ne)
Gi = G.subgroup(Ni)

G.v = c
G.u = b * c
Ge.a = a_e
Ge.d = d_e
Gi.a = a_i
Gi.d = d_i

# In Izhikevich's simulation, the value of I is set to zero each time step,
# then a 'thalamic input' is added to I, then spikes cause a postsynaptic
# effect on I, and then the numerical integration step is performed. In
# Brian, the numerical integration step is performed before spike propagation,
# so we insert the thalamic input after the numerical integration but before
# the spike propagation, using the when='before_connections' setting.

change_every = int(1*ms/dt)
change_time = 0
thal_ind = 0
@network_operation(when='before_connections')
def thalamic_input():
    global change_time, thal_ind
    G.I = 0 * mV / ms
    G.I[thal_ind] = float(thalamic_input_weight)
    if change_time==0:
        change_time = change_every
        thal_ind = randint(N)
        #G.I[randint(N)] = float(thalamic_input_weight*1*ms/dt)
    change_time -= 1

Ce = Connection(Ge, G, 'I', delay=True, max_delay=D)
Ci = Connection(Gi, Ge, 'I', delay=D_i)

# In order to implement Izhikevich's STDP rule, we need two weight matrices,
# one is the actual weight matrix and the second is a weight matrix derivative.
# To do this with Brian's sparse matrices, we create a second Connection
# Ce_deriv, which we initialise to have the same shape as Ce, but we use the
# forget() function on the Connection so that although the object exists,
# no spikes will be propagated by that Connection. We need the object to exist
# because we create an ExponentialSTDP object that acts on Ce_deriv not
# directly on Ce.

Ce_deriv = Connection(Ge, G, 'I', delay=True, max_delay=D)
forget(Ce_deriv)

for i in xrange(Ne):
    inds = random.sample(xrange(N), M)
    Ce[i, inds] = s_e * ones(len(inds))
    Ce.delay[i, inds] = rand(len(inds)) * (D - D_min) + D_min
for i in xrange(Ni):
    inds = random.sample(xrange(Ne), M)
    Ci[i, inds] = s_i * ones(len(inds))

# Now we duplicate Ce to Ce_deriv

for i in xrange(Ne):
    Ce_deriv[i, :] = Ce[i, :]
    Ce_deriv.delay[i, :] = Ce.delay[i, :]

# STDP acts directly on Ce_deriv rather than Ce. With this STDP rule alone,
# we would not see any learning, the network operation below propagates changes
# in Ce_deriv to Ce.

artificial_wmax = 1e10 * mV / ms
stdp = ExponentialSTDP(Ce_deriv,
                       taup=20 * ms, taum=20 * ms,
                       Ap=(0.1 * mV / ms / artificial_wmax),
                       Am=(-0.12 * mV / ms / artificial_wmax),
                       wmin= -artificial_wmax,
                       wmax=artificial_wmax,
                       interactions='nearest'
                       )

# Izhikevich's STDP rule has STDP acting on a matrix sd of derivatives, and
# then each second the weight matrix s and derivates sd are updated according
# to the rule:
#   s <- s+0.01+sd
#   sd <- sd*0.9
# Note also that we are using Brian's sparse matrices, but they are have
# exactly the same nonzero entries, and therefore we can do arithmetic
# operations on these matrices using the alldata attribute of Brian sparse
# matrices. The compress() method converts the ConstructionMatrix into a
# ConnectionMatrix, thus freezing the nonzero entries. In the next line, we
# want the actual starting values of Ce_deriv to be zero, but if we had done
# this before compressing to a ConnectionMatrix, all entries would be
# considered zero entries in the sparse matrix, and then Ce and Ce_deriv would
# have a different pattern of non-zero entries.

Ce_deriv.compress()
Ce_deriv.W.alldata[:] = 0
@network_operation(clock=EventClock(dt=1 * second))
def update_weights_from_derivative():
    Ce.W.alldata[:] = clip(Ce.W.alldata + 0.01 * mV / ms + Ce_deriv.W.alldata, 0, sm)
    Ce_deriv.W.alldata[:] *= 0.9

M = SpikeMonitor(G)

ion()
raster_plot(M, refresh=.2 * second, showlast=1 * second)

run(50 * second, report='stderr')

ioff()
show()



from numpy import *
from pylab import *
import pickle
import os
import itertools
import cuba_runopts

basedir = 'data'

def get_packaged_results(packagename, var, desc):
    if basedir: packagename = basedir + '/' + packagename
    exec open(packagename + '.py').read()
    res = locals()[var]
    return (res, desc)

def get_brian(ext, desc, runtype=''):
    if ext:
        ext = '_' + ext
    if runtype:
        runtype = '_' + runtype
    pre = ''
    if basedir: pre = basedir + '/'
    inputfile = open(pre + 'brian_cuba' + runtype + '_results' + ext + '.pkl', 'rb')
    brian_cuba = pickle.load(inputfile)
    inputfile.close()
    return (brian_cuba, desc)

def plotresults(res, desc):
    N, t, spikes, results = zip(*res)
    R = array(results)
    tstd = std(R, axis=1)[:, 0]
    errorbar(N, t, yerr=tstd, label=desc)

def plotcomparativeresults(base, res, desc):
    Nb, tb, spikesb, resultsb = zip(*base)
    N, t, spikes, results = zip(*res)
    tb = array(tb)[:len(N)]
    plot(N, array(t) / tb, '-o', label=desc)

def plotrates(res, desc):
    N, t, spikes, results = zip(*res)
    R = array(results)
    tstd = std(R, axis=1)[:, 1]
    errorbar(N, spikes, yerr=tstd, label=desc)

def plotvaryconresults(res, desc):
    N, t, spikes, results = zip(*res)
    results = array(results)
    results[:, :, 1] /= N[0] * cuba_runopts.duration
    spikes = array(spikes) / (N[0] * cuba_runopts.duration)
    for k, n in zip(results, N):
        R = array(k)
        plot(R[:, 1], R[:, 0], '.', label=desc)
    R = array(results)
    ststd = std(R, axis=1)
    sstd = ststd[:, 1]
    tstd = ststd[:, 0]
    errorbar(spikes, t, tstd, sstd, label=desc)

def plotvaryweresults(res, desc):
    N, t, spikes, results = zip(*res)
    results = list(itertools.chain(*results))
    results = array(results)
    results[:, 1] /= N[0] * cuba_runopts.duration
    plot(results[:, 1], results[:, 0], '.', label=desc)

def do_analysis(all_results, showrates=True):
    cpp_cuba = all_results[0][0]
    cpp_cuba_desc = all_results[0][1]
    N, t, spikes, results = zip(*cpp_cuba)
    Nmax = max(N)
    Nmin = min(N)

    d1 = 1
    if showrates:
        d2 = 3
    else:
        d2 = 2

    figure()

    subplot(d1, d2, 1)
    for r, desc in all_results:
        plotresults(r, desc)
    #legend([desc for r,desc in all_results])
    #legend(loc='upper left')
    axis(ymin=0, xmin=Nmin, xmax=Nmax)
    xlabel('N')
    ylabel('Time (s)')

    subplot(d1, d2, 2)
    for r, desc in all_results:
        plotcomparativeresults(cpp_cuba, r, desc)
    #legend([desc for r,desc in all_results])
    #legend(loc='upper left')
    legend()
    axis(ymin=0, xmin=Nmin, xmax=Nmax)
    xlabel('N')
    ylabel('Time relative to ' + cpp_cuba_desc + ' (s)')

    if showrates:
        subplot(d1, d2, 3)
        for r, desc in all_results:
            plotrates(r, desc)
        axis(ymin=0, xmin=Nmin, xmax=Nmax)
        xlabel('N')
        ylabel('Num. spikes')

#    if showrates:
#        subplot(d1,d2,4)
#    else:
#        subplot(d1,d2,3)
#    figlegend()

#    show()

def do_analysis_varywe(all_results):
    figure()
    for res, desc in all_results:
        plotvaryweresults(res, desc)
    xlabel('Firing rate per neuron (Hz)')
    ylabel('Time (s)')
    legend(loc='upper left')
#    do_analysis(all_results)

def cd(s):
    global basedir
    basedir = s

if __name__ == '__main__':
    all_results = []
    all_results.append(get_packaged_results('cpp_cuba_results_matrix', 'cpp_cuba', 'C++ (gen. matrix)'))
    all_results.append(get_packaged_results('cpp_cuba_results', 'cpp_cuba', 'C++ (Euler)'))
    all_results.append(get_packaged_results('cpp_cuba_results_matrix_efficient', 'cpp_cuba', 'C++ (fix. matrix)'))
    all_results.append(get_packaged_results('matlab_cuba_results', 'matlab_cuba', 'Matlab'))
    all_results.append(get_brian('compile_nounits', 'Brian compiled, no units'))
    do_analysis(all_results, showrates=False)

    all_results = []
    all_results.append(get_packaged_results('cpp_cuba_nospiking_results_matrix', 'cpp_cuba', 'C++ (gen. matrix)'))
    all_results.append(get_packaged_results('cpp_cuba_nospiking_results', 'cpp_cuba', 'C++ (Euler)'))
    all_results.append(get_packaged_results('matlab_cuba_nospiking_results', 'matlab_cuba', 'Matlab'))
    all_results.append(get_brian('compile_nounits', 'Brian compiled, no units', 'nospiking'))
    do_analysis(all_results, showrates=False)

#    all_results = []
#    all_results.append(get_packaged_results('cpp_cuba_varywe_results_matrix','cpp_cuba','C++ (gen. matrix)'))
#    all_results.append(get_packaged_results('cpp_cuba_varywe_results','cpp_cuba','C++ (Euler)'))
#    all_results.append(get_packaged_results('matlab_cuba_varywe_results','matlab_cuba','Matlab'))
#    all_results.append(get_brian('compile_nounits_1000','Brian','varywe'))
#    do_analysis_varywe(all_results)

    all_results = []
    all_results.append(get_packaged_results('cpp_cuba_varywe_results_matrix_4000', 'cpp_cuba', 'C++ (gen. matrix)'))
    all_results.append(get_packaged_results('matlab_cuba_varywe_results_4000', 'matlab_cuba', 'Matlab'))
    all_results.append(get_brian('compile_nounits_4000', 'Brian', 'varywe'))
    do_analysis_varywe(all_results)

    show()



import os, glob
curdir = os.getcwd()
#print "Generating interface..."
#os.system('generate_interface.py')
#os.chdir(curdir)
print "Generating tutorials..."
os.system('generate_tutorials.py')
os.chdir(curdir)
print "Generating preference list..."
os.system('generate_prefsdocs.py')
os.chdir(curdir)
print "Generating examples..."
os.system('generate_examples.py')
os.chdir(curdir)
print "Generating HTML..."
# use clean build because this script will typically be called for new releases only
import build_html_clean
print "Done"



'''
Utility functions adapted from MAP 
'''
from brian import *

__all__ = ['erbspace']

@check_units(low=Hz, high=Hz)
def erbspace(low, high, N, earQ=9.26449, minBW=24.7, order=1):
    '''
    Returns the centre frequencies on an ERB scale.
    
    ``low``, ``high``
        Lower and upper frequencies
    ``N``
        Number of channels
    ``earQ=9.26449``, ``minBW=24.7``, ``order=1``
        Default Glasberg and Moore parameters.
    '''
    low = float(low)
    high = float(high)
    cf = -(earQ * minBW) + exp((arange(N)) * (-log(high + earQ * minBW) + \
            log(low + earQ * minBW)) / (N-1)) * (high + earQ * minBW)
    cf = cf[::-1]
    return cf

# Testing
if __name__ == '__main__':
    cf = erbspace(20 * Hz, 20 * kHz, 3000)
    print amin(cf), amax(cf)
    print diff(cf)[-5:]
    plot(cf)
    show()



#!/usr/bin/env python

"""
setup.py file for SWIG example
"""

from distutils.core import setup, Extension


test_module = Extension('_test',
                           sources=['test_wrap.c', 'test.c'],
                           )

setup (name='test',
       version='0.1',
       author="SWIG Docs",
       description="""Simple swig example from docs""",
       ext_modules=[test_module],
       py_modules=["test"],
       )



import brian_no_units
from brian import *
reinit()

#RS
a = 0.02
b = 0.2
c = -65
d = 8


#---------------------------------------------------------------------------------

defaultclock.dt = 0.001

n = 2 #number of neurons in the network
r = 20 #strength of external input



eqs = Equations(''' dv/dt= 0.04*v**2 + 5*v + 140 - u + I: 1
du/dt=a*(b*v-u):1
I:1
''')


PG = PoissonGroup(n, rates=2 * Hz)

G = NeuronGroup (n, model=eqs, reset="v=c;u=u+d", threshold='v>=30')



@network_operation(when='after_groups')
def Icurrent():
    G.I = 10


C2 = IdentityConnection(PG, G, 'I', weight=r)

G.v = -65
G.u = b * G.v
G.I = 10

Sp = SpikeMonitor(G)
M = MultiStateMonitor(G, record=True)



net = Network(Icurrent, PG, G, C2, Sp, M)


net.run(140)



figure(1)
raster_plot(Sp)


figure(2)
subplot(4, 1, 1)
plot(M['v'].times, M['v'][0])
ylabel('v (dimensionless)')
subplot(4, 1, 2)
plot(M['I'].times, M['I'][0])
ylabel('I (dimensionless)')
subplot(4, 1, 3)
plot(M['u'].times, M['u'][0])
xlabel('Time/seconds')
ylabel('u (dimensionless)')


figure(3)
subplot(4, 1, 1)
plot(M['v'].times, M['v'][1])
ylabel('v (dimensionless)')
subplot(4, 1, 2)
plot(M['I'].times, M['I'][1])
ylabel('I (dimensionless)')
subplot(4, 1, 3)
plot(M['u'].times, M['u'][1])
xlabel('Time/seconds')
ylabel('u (dimensionless)')


show()



from ..connections.base import *
from ..connections.sparsematrix import *
from ..connections.connectionvector import *
from ..connections.constructionmatrix import *
from ..connections.connectionmatrix import *
from ..connections.construction import *
from ..connections.connection import *
from ..connections.delayconnection import *
from ..connections.propagation_c_code import *
import warnings
import numpy.random
network_operation = None # we import this when we need to because of order of import issues

__all__ = [
          'StochasticConnection',
          ]


class StochasticConnection(Connection):
    '''
    Connection which implements probabilistic (unreliable) synapses
    
    Initialised as for a :class:`Connection`, but with the additional
    keyword:
    
    ``reliability``
        Specifies the probability that a presynaptic spike triggers
        a postsynaptic current in the postsynaptic neuron.
    '''
    @check_units(delay=second)
    def __init__(self, source, target, state=0, delay=0 * msecond,
                 structure='dense',
                 reliability=1.0, **kwds):
        Connection.__init__(self, source, target, state=state, delay=delay,
                            structure=structure, **kwds)
        if (len(source) != len(target)):
            raise AttributeError, 'The connected (sub)groups must have the same size.'
        source.set_max_delay(delay)
        self.reliability = reliability

    def propagate(self, spikes):
        '''
        Propagates the spikes to the target.
        '''
        if len(spikes):
            rows = self.W[spikes, :] * (numpy.random.rand(len(spikes), self.W.shape[1]) <= self.reliability)
            for row in rows:
                self.target._S[self.nstate, :] += array(row, dtype=float32)

    def compress(self):
        pass

            


'''
A network of exponential IF models with synaptic conductances
'''
from brian import *
from brian.library.IF import *
from brian.library.synapses import *
import time

C = 200 * pF
taum = 10 * msecond
gL = C / taum
EL = -70 * mV
VT = -55 * mV
DeltaT = 3 * mV

# Synapse parameters
Ee = 0 * mvolt
Ei = -80 * mvolt
taue = 5 * msecond
taui = 10 * msecond

eqs = exp_IF(C, gL, EL, VT, DeltaT)
# Two different ways of adding synaptic currents:
eqs += Current('''
Ie=ge*(Ee-vm) : amp
dge/dt=-ge/taue : siemens
''')
eqs += exp_conductance('gi', Ei, taui) # from library.synapses

P = NeuronGroup(4000, model=eqs, threshold= -20 * mvolt, reset=EL, refractory=2 * ms)
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
we = 1.5 * nS # excitatory synaptic weight
wi = 2.5 * we # inhibitory synaptic weight
Ce = Connection(Pe, P, 'ge', weight=we, sparseness=0.05)
Ci = Connection(Pi, P, 'gi', weight=wi, sparseness=0.05)
# Initialization
P.vm = randn(len(P)) * 10 * mV - 70 * mV
P.ge = (randn(len(P)) * 2 + 5) * we
P.gi = (randn(len(P)) * 2 + 5) * wi

# Excitatory input to a subset of excitatory and inhibitory neurons
# Excitatory neurons are excited for the first 200 ms
# Inhibitory neurons are excited for the first 100 ms
input_layer1 = Pe.subgroup(200)
input_layer2 = Pi.subgroup(200)
input1 = PoissonGroup(200, rates=lambda t: (t < 200 * ms and 2000 * Hz) or 0 * Hz)
input2 = PoissonGroup(200, rates=lambda t: (t < 100 * ms and 2000 * Hz) or 0 * Hz)
input_co1 = IdentityConnection(input1, input_layer1, 'ge', weight=we)
input_co2 = IdentityConnection(input2, input_layer2, 'ge', weight=we)

# Record the number of spikes
M = SpikeMonitor(P)

print "Simulation running..."
start_time = time.time()
run(500 * ms)
duration = time.time() - start_time
print "Simulation time:", duration, "seconds"
print M.nspikes / 4000., "spikes per neuron"
raster_plot(M)
show()



'''
Example of how to use the van Rossum metric. 

The VanRossumMetric function, which is defined as a monitor and therefore works online, 
computes  the metric between every neuron in a given population. The present example show 
the concept of phase locking:  N neurons  are driven by  sinusoidal inputs with different amplitude.

 Use: output=VanRossumMetric(source, tau=4 * ms)
 
 source is a NeuronGroup of N neurons
 tau is the time constant of the kernel used in the metric
 
 output is a monitor with attribute distance which is the distance matrix between the neurons in source
'''
from brian import *
from time import time

tau=20*ms
N=100
b=1.2 # constant current mean, the modulation varies
f=10*Hz
delta =2*ms

eqs='''
dv/dt=(-v+a*sin(2*pi*f*t)+b)/tau : 1
a : 1
'''

neurons=NeuronGroup(N,model=eqs,threshold=1,reset=0)
neurons.v=rand(N)
neurons.a=linspace(.05,0.75,N)
S=SpikeMonitor(neurons)
trace=StateMonitor(neurons,'v',record=50)

van_rossum_metric=VanRossumMetric(neurons, tau=4 * ms)

run(1000*ms)

raster_plot(S)
title('Raster plot')

figure()
title('Distance matrix between spike trains')
imshow(van_rossum_metric.distance)
colorbar()
show()


from brian import *
from nose.tools import *

def test():
    """
    See the main documentation or the API documentation for details of the
    purpose, main functions and classes of the magic module.
    
    Functions
    ~~~~~~~~~
    
    * :func:`get_instances(instancetype,level=1)`
    * :func:`find_instances(instancetype,startlevel=1)`
    * :func:`find_all_instances(instancetype,startlevel=1)`
    
    Here instancetype is a class derived from :class:`InstanceTracker`, including
    :class:`Clock`, :class:`NeuronGroup`, :class:`Connection`, :class:`NetworkOperation`.
    ``level`` is an integer
    greater than 0 that tells the function how far back it should search
    in the sequence of called functions. ``level=0`` means it should find
    instances from the function calling one of the magic functions,
    ``level=1`` means it should find instances from the function calling the
    function calling one of the magic functions, etc.
    
    :func:`get_instances` returns all instances at a specified level.
    :func:`find_instances`
    searches increasing levels starting from the given ``startlevel`` until it
    finds a nonzero number of instances, and then returns those.
    :func:`find_all_instances` finds all instances from a given level onwards.
    
    Classes
    ~~~~~~~
    
    An object of a class ``cls`` derived from ``InstanceTracker`` will be tracked
    if ``cls._track_instances()`` returns ``True``. The default behaviour of
    ``InstanceTracker`` is to always return ``True``, but this method can be
    redefined if you do not want to track instances of a particular
    subclass of a tracked class. Note that this method is a static method
    of a class, and cannot be used to stop particular instances being
    tracked. Redefine it using something like::
        
        class A(InstanceTracker):
            pass
        class B(A):
            @staticmethod
            def _track_instances(): return False

    A warning (technical detail)
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    The current implementation of instance tracking will return
    every variable that was created in a given execution frame (i.e. in
    a given function) that is still alive (i.e. a reference still exists
    to it somewhere). So for example, the following will not find any
    instance::
    
        class A(InstanceTracker):
            pass
        def f(A):
            return A()
        a = f(A)
        print get_instances(A,level=0)
    
    The reason is that the object a was created in the function ``f``, but
    the :func:`get_instances` statement is run in the main body. Similarly, the
    following will return two instances rather than one as you might
    expect::
    
        class A(InstanceTracker):
            pass
        def f(A):
            a = A()
            return get_instances(A,level=0)
        insts1 = f(A)
        isnts2 = f(A)
        print insts2
    
    The reason is that the first call to ``f`` defines an instance of ``A`` and
    returns (via :func:`get_instances`) an object which contains a reference to it.
    The object is still therefore alive. The second call to ``f`` creates a new
    ``A`` object, but the :func:`get_instances` call returns both ``A`` objects, because they
    are both still alive (the first is stored in ``insts1``) and both created
    in the function ``f``.
    
    This behaviour is not part of the assured interface of the magic module,
    and you shouldn't rely on it.
    """
    reinit_default_clock()

    # Define a heirarchy of classes A, B, C, D, E and track instances of them
    # Do not track instances of D or E
    class A(InstanceTracker):
        gval = 0
        def __init__(self):
            self.value = A.gval
            A.gval += 1
        def __repr__(self):
            return str(self)
        def __str__(self):
            return str(self.value)
    class B(A):
        pass
    class C(B):
        pass
    class D(C):
        @staticmethod
        def _track_instances(): return False
    class E(D):
        pass

    # Create some sample objects of each type
    a1 = A() # object 0
    a2 = A() # object 1
    b = B()  # object 2
    c = C()  # object 3
    d = D()  # object 4
    e = E()  # object 5

    # Find the instances of each type on this level
    instA, names = get_instances(A, level=0)
    instB, names = get_instances(B, level=0)
    instC, names = get_instances(C, level=0)
    instD, names = get_instances(D, level=0)
    instE, names = get_instances(E, level=0)

    # This is the expected behaviour:
    # instA = [a1, a2, b, c]
    assert all(o in instA for o in [a1, a2, b, c])
    assert all(o not in instA for o in [d, e])
    # instB = [b, c]
    assert all(o in instB for o in [b, c])
    assert all(o not in instB for o in [a1, a2, d, e])
    # instC = [c]
    assert c in instC
    assert all(o not in instC for o in [a1, a2, b, d, e])
    # instD = instE = []
    assert len(instD) == 0
    assert len(instE) == 0

    # Check that level=0 and level=1 work as expected
    def f1(vars, A, B, C, D, E):
        a3 = A() # object 6
        inst_ahere, names = get_instances(A, level=0) # level=0 should refer to definitions inside f
        inst_abefore, names = get_instances(A, level=1) # level=1 should refer to definitions inside the function calling f
        # inst_abefore = [a1, a2, b, c]
        assert all(o in instA for o in vars[0:4])
        assert all(o not in instA for o in vars[4:])
        # inst_ahere = [a3]
        assert len(inst_ahere) == 1 and a3 in inst_ahere
    f1([a1, a2, b, c, d, e], A, B, C, D, E)

    # Check that nested function calling works as expected
    def f2(A):
        a4 = A() # object 7
        return [get_instances(A, level) for level in range(2)]
    inst = f2(A)
    # inst[0][0] = [a4]
    assert str(inst[0][0]) == '[7]'
    # inst[1][0] = [a1,a2,c,b]
    assert len(inst[1][0]) == 4 and all(o in inst[1][0] for o in [a1, a2, b, c])
    def f3(A):
        a5 = A() # object 9
        return [get_instances(A, level) for level in range(2)]
    def f4(A):
        a6 = A() # object 8
        return f3(A)
    inst = f4(A)
    # inst[0][0] = [a5]
    assert str(inst[0][0]) == '[9]'
    # inst[1][0] = [a6]
    assert str(inst[1][0]) == '[8]'

    # check that find_instances works as expected
    def f5(A):
        return f6(A)
    def f6(A):
        a = A() # object 10
        return f7(A)
    def f7(A):
        return find_instances(A, startlevel=0)
    inst = f5(A)[0][0]
    assert str(inst) == '10'

    # check that find_all_instances works as expected
    def f8(A):
        a = A() # object 11
        return find_all_instances(A, startlevel=0)
    insts = f8(A)[0] # should be objects 0,1,2,3 and 11
    s = map(str, insts)
    s.sort()
    assert str(s) == "['0', '1', '11', '2', '3']"

if __name__ == '__main__':
    test()



from brian import *

__all__ = ['Group', 'MultiGroup']


class Group(object):
    '''
    Generic fixed-length variable values container class
    
    Used internally by Brian to store values associated with variables. Primary
    use is for NeuronGroup to store a state matrix of values associated to the
    different variables.
    
    Each differential equation in ``equations`` is allocated an array of
    length ``N`` in the attribute ``_S`` of the object. Unit consistency
    checking is performed.
    '''
    def __init__(self, equations, N, level=0, unit_checking=True):
        if isinstance(equations, str):
            equations = Equations(equations, level=level + 1)
        equations.prepare(check_units=unit_checking)
        var_names = equations._diffeq_names
        M = len(var_names)
        self._S = zeros((M, N))
        self.staticvars = dict([(name, equations._function[name]) for name in equations._eq_names])
        self.var_index = dict(zip(var_names, range(M)))
        self.var_index.update(zip(range(M), range(M))) # name integer i -> state variable i
        for var1, var2 in equations._alias.iteritems():
            self.var_index[var1] = self.var_index[var2]

    def get_var_index(self, name):
        '''
        Returns the index of state variable "name".
        '''
        return self.var_index[name]

    def __len__(self):
        '''
        Number of neurons in the group.
        '''
        return self._S.shape[1]

    def num_states(self):
        return self._S.shape[0]

    def state_(self, name):
        '''
        Gets the state variable named "name" as a reference to the underlying array
        '''
        if isinstance(name, int):
            return self._S[name]
        if name in self.staticvars:
            f = self.staticvars[name]
            return f(*[self.state_(var) for var in f.func_code.co_varnames])
        i = self.var_index[name]
        return self._S[i]
    state = state_

    def __getattr__(self, name):
        if name == 'var_index':
            # this seems mad - the reason is that getattr is only called if the thing hasn't
            # been found using the standard methods of finding attributes, which for var_index
            # should have worked, this is important because the next line looks for var_index
            # and if we haven't got a var_index we don't want to get stuck in an infinite
            # loop
            raise AttributeError
        if not hasattr(self, 'var_index'):
            # only provide lookup of variable names if we have some variable names, i.e.
            # if the var_index attribute exists
            raise AttributeError
        try:
            return self.state(name)
        except KeyError:
            if len(name) and name[-1] == '_':
                try:
                    origname = name[:-1]
                    return self.state_(origname)
                except KeyError:
                    raise AttributeError
            raise AttributeError

    def __setattr__(self, name, val):
        origname = name
        if len(name) and name[-1] == '_':
            origname = name[:-1]
        if not hasattr(self, 'var_index') or (name not in self.var_index and origname not in self.var_index):
            object.__setattr__(self, name, val)
        else:
            if name in self.var_index:
                self.state(name)[:] = val
            else:
                self.state_(origname)[:] = val


class MultiGroup(object):
    '''
    Generic variable-length variable values container class
    
    Used internally by Brian to store values associated with variables.
    See :class:`Group` for more details.
    
    This class takes a list of groups or of ``(equations, N)`` pairs.
    The ``state()`` and ``state_()`` methods work as if it were a
    single :class:`Group` but return arrays of different lengths from
    different state matrix structures depending on where the state
    value is defined. Similarly for attribute access.
    '''
    def __init__(self, groups, level=0):
        newgroups = []
        self.var_index = dict()
        for g in groups:
            if not isinstance(g, Group):
                eqs, N = g
                g = Group(eqs, N, level=level + 1)
            newgroups.append(g)
            self.var_index.update(dict((k, g) for k in g.var_index.keys()))
            self.var_index.update(dict((k, g) for k in g.staticvars.keys()))
        self.groups = newgroups

    def state_(self, name):
        return self.var_index[name].state_(name)
    state = state_

    def __getattr__(self, name):
        try:
            return self.state(name)
        except KeyError:
            if len(name) and name[-1] == '_':
                try:
                    origname = name[:-1]
                    return self.state_(origname)
                except KeyError:
                    raise AttributeError
            raise AttributeError

    def __setattr__(self, name, val):
        origname = name
        if len(name) and name[-1] == '_':
            origname = name[:-1]
        if not hasattr(self, 'var_index') or (name not in self.var_index and origname not in self.var_index):
            object.__setattr__(self, name, val)
        else:
            if name in self.var_index:
                self.state(name)[:] = val
            else:
                self.state_(origname)[:] = val

if __name__ == '__main__':
    if True:
        eqs_A = Equations('''
            dV/dt = -V/(1*second) : volt
            W = V*V : volt2
            x = V
            ''')
        eqs_B = Equations('''
            dVb/dt = -Vb/(1*second) : volt
            Wb = Vb*Vb : volt2
            xb = Vb
            ''')
        g = MultiGroup(((eqs_A, 10), (eqs_B, 5)))
        for h in g.groups:
            h._S[:] = rand(*h._S.shape)
        print g.V
        print g.V ** 2
        print g.W
        print g.x
        print g.Vb
        print g.Vb ** 2
        print g.Wb
        print g.xb
    if False:
        eqs = Equations('''
            dV/dt = -V/(1*second) : volt
            W = V*V : volt2
            x = V
            ''')
        g = Group(eqs, 10)
        print g._S
        g._S[:] = rand(*g._S.shape)
        print g._S
        print g.V
        print g.V ** 2
        print g.W
        print g.x
        g.V = -1
        print g.V
        print g.W
        print g.x



import os

# delete this when make_new_release.py is working
exclude = [ 'autodiff' ]

def api_command_line(exclude):
    s = 'epydoc.py -o docs/api --docformat plaintext --no-private --no-frames --no-sourcecode '
    if len(exclude):
        s += '--exclude="' + '|'.join(exclude) + '" '
    s += 'brian'
    return s

if __name__ == '__main__':
    basepathname, filename = os.path.split(__file__)
    os.chdir(basepathname)
    os.chdir('../../../.') # work from root directory
    os.system(api_command_line(exclude))



from brian import *
from brian.library.ionic_currents import *
from IPython.Shell import IPShellEmbed
ipshell = IPShellEmbed()

N = 100
duration = 1 * second

El = 10.6 * mV
EK = -12 * mV
ENa = 120 * mV
eqs = MembraneEquation(1 * uF) + leak_current(.3 * msiemens, El)
eqs += K_current_HH(36 * msiemens, EK) + Na_current_HH(120 * msiemens, ENa)
eqs += Current('I:amp')
neuron = NeuronGroup(N, eqs, implicit=True, freeze=True)
trace = RecentStateMonitor(neuron, 'vm', record=[0, 1, 2], duration=50 * ms)
neuron.I = 10 * uA * rand(N)

ion()
trace.plot(refresh=1 * ms)
net = MagicNetwork()

while defaultclock.t < duration:
    if hasattr(net, 'stopped') and net.stopped:
        break
    try:
        print 'Starting at time', defaultclock.t
        net.run(duration - defaultclock.t)
    except KeyboardInterrupt:
        print 'Interrupted at time', defaultclock.t
        ipshell(local_ns=locals(), global_ns=globals())
print 'Finished at time', defaultclock.t
ioff()
clf()
trace.plot()
show()



def convolution_matrix(src, dst, kernel, toric=False):
    '''
    Build a sparse convolution matrix M such that:

    (M*src.ravel()).reshape(src.shape) = convolve2d(src,kernel)

    You can specify whether convolution is toric or not and specify a different
    output shape. If output (dst) is different, convolution is only applied at
    corresponding normalized location within the src array.

    Building the matrix can be pretty long if your kernel is big but it can
    nonetheless saves you some time if you need to apply several convolution
    compared to fft convolution (no need to go to the Fourier domain).

    Parameters:
    -----------

    src : n-dimensional numpy array
        Source shape

    dst : n-dimensional numpy array
        Destination shape

    kernel : n-dimensional numpy array
        Kernel to be used for convolution

    Returns:
    --------

    A sparse convolution matrix

    Examples:
    ---------

    >>> Z = np.ones((3,3))
    >>> M = convolution_matrix(Z,Z,Z,True)
    >>> print (M*Z.ravel()).reshape(Z.shape)
    [[ 9.  9.  9.]
     [ 9.  9.  9.]
     [ 9.  9.  9.]]
    >>> M = convolution_matrix(Z,Z,Z,False)
    >>> print (M*Z.ravel()).reshape(Z.shape)
    [[ 4.  6.  4.]
     [ 6.  9.  6.]
     [ 4.  6.  4.]]
    '''
 
    # For a toric connection, it is wrong to have a kernel larger
    # than the source
 #   if toric:
 #       shape = np.minimum(np.array(src.shape), np.array(kernel.shape))
 #       kernel = extract(kernel, shape, np.rint(np.array(kernel.shape)/2.))

    # Get non NaN value from kernel and their indices.
    nz = (1 - np.isnan(kernel)).nonzero()
    data = kernel[nz].ravel()
    indices = [0,]*(len(kernel.shape)+1)
    indices[0] = np.array(nz)
    indices[0] += np.atleast_2d((np.array(src.shape)//2 - np.array(kernel.shape)//2)).T

    # Generate an array A for a given shape such that given an index tuple I,
    # we can translate into a flat index F = (I*A).sum()
    to_flat_index = np.ones((len(src.shape),1), dtype=int)
    if len(src.shape) > 1:
        to_flat_index[:-1] = src.shape[1]

    R, C, D = [], [], []
    dst_index = 0
    src_indices = []

    # Translate target tuple indices into source tuple indices taking care of
    # possible scaling (this is done by normalizing indices)
    for i in range(len(src.shape)):
        z = np.rint((np.linspace(0,1,dst.shape[i])*(src.shape[i]-1))).astype(int)
        src_indices.append(z)

    nd = [0,]*(len(kernel.shape))
    for index in np.ndindex(dst.shape):
        dims = []
        # Are we starting a new dimension ?
        if index[-1] == 0:
            for i in range(len(index)-1,0,-1):
                if index[i]: break
                dims.insert(0,i-1)
        dims.append(len(dst.shape)-1)
        for dim in dims:
            i = index[dim]

            if toric:
                z = (indices[dim][dim] - src.shape[dim]//2 +(kernel.shape[dim]+1)%2 + src_indices[dim][i]) % src.shape[dim]
            else:
                z = (indices[dim][dim] - src.shape[dim]//2 +(kernel.shape[dim]+1)%2 + src_indices[dim][i])

            # if toric:
            #     z = (indices[dim][dim] - src.shape[dim]/2.0 -(kernel.shape[dim]+1)%2 + src_indices[dim][i]) % src.shape[dim]
            # else:
            #     z = (indices[dim][dim] - src.shape[dim]/2.0 -(kernel.shape[dim]+1)%2+ src_indices[dim][i])

            n = np.where((z >= 0)*(z < src.shape[dim]))[0]
            if dim == 0:
                nd[dim] = n.copy()
            else:
                nd[dim] = nd[dim-1][n]
            indices[dim+1] = np.take(indices[dim], n, 1)
            indices[dim+1][dim] = z[n]
        dim = len(dst.shape)-1
        z = indices[dim+1]
        R.extend( [dst_index,]*len(z[0]) )
        C.extend( (z*to_flat_index).sum(0).tolist() )
        D.extend( data[nd[-1]].tolist() )
        dst_index += 1

    return sp.coo_matrix( (D,(R,C)), (dst.size,src.size))



def importSpkTrain(Nout,file_name="embeddedInputs.mat",var="embeddedInputs"):
    global InhNeur, ExNeur, SpkTrainEx, SpkTrainInh
    cellImport = scipy.io.loadmat(file_name, matlab_compatible=True)
    inputSpkTimes = cellImport[var]
        
    allSpkTrain=[]
    allSpkTrainImport=[]
    for j in range(Nout):
        spkTimeTemp=[]
        for ind1 in range(len(inputSpkTimes[j,inputInstance-1])):
            spkTimeTemp.append(Quantity(inputSpkTimes[j,inputInstance-1][ind1][0])*ms)
    
        allSpkTrain.append(spkTimeTemp)
    
    shuffleNeur=random.permutation(Nout) # implement 12.5 or 25 % of inhibition
    #InhNeur = shuffleNeur[:Nout/4]
    InhNeur = shuffleNeur[:Nout/16] # 16
    #ExNeur =  shuffleNeur[Nout/4:len(shuffleNeur)]
    ExNeur =  shuffleNeur[Nout/16:len(shuffleNeur)] #16
    
    SpkTrainEx=[]
    for indEx in range(len(ExNeur)):
        SpkTrainEx.append(allSpkTrain[ExNeur[indEx]])
        
    SpkTrainInh=[]
    for indInh in range(len(InhNeur)):
        SpkTrainInh.append(allSpkTrain[InhNeur[indInh]])
    
    return InhNeur, ExNeur, SpkTrainEx, SpkTrainInh

def Wdefinition(Nout,ExNeur,InhNeur):
    global W_ex, W_inh
    W=ones((Nout,Nout))  
    W_ex=W[:len(ExNeur),:]
    W_inh=W[len(ExNeur):,:]
    
    return W_ex, W_inh

def importDelays(file_name2="delta_ij.mat",var2="delta_ij"):
    global delta_ij
    d = scipy.io.loadmat(file_name2, matlab_compatible=True, mdict=None)
    delta_ij=d[var2]
    
    return delta_ij

#-------------------------------------------------------------------------------
from brian import *
from numpy import *
from scipy import *
import scipy.io
from scipy.stats import poisson
from numpy.random import *

simclock = Clock(dt=0.001 *ms)
monclock = Clock(dt=0.1 *ms)

patch_size = 8 #

numPix=patch_size*patch_size
Nout = numPix

inputInstance = 3       #  

C_m = 0.5 * nF
tau_m = 20 * ms

Vt = -50 * mV		# spike threshold | 1 * mV
Vr = -60 * mV		# resting potential and reset value | 0 * mV
stdVr = 3               # std of Vr

E_L = -70 * mV          # resting potential | 0 * mV
gL = 100                # leak conductance 100 * nS
     
E_ex = 0 * mV           # excitatory reverse potential
#g_ex = 25 * nS
tau_ex = 1 * ms         # 5 * ms

E_inh = -70 * mV        # inhibitory reverse potential
#g_inh = 20 * nS
tau_inh = 1 * ms       # 10 * mS

ref = 3 * ms            # neurons' refractory period

# COBA model
eqs = Equations('''
        dV/dt = (-gL*(V-E_L)-g_ex*(V-E_ex)-g_inh*(V-E_inh))*(1./C_m)    : volt
        dg_ex/dt = -g_ex/tau_ex                                         : nS
        dg_inh/dt = -g_inh/tau_inh                                      : nS
''')

#eqs = Equations('''
#        dV/dt = (-gL*(V-E_L)-g_ex*(V-E_ex)-g_inh*(V-E_inh))*(1./tau_m)    : volt
#        dg_ex/dt = -g_ex/tau_ex                                           : 1
#        dg_inh/dt = -g_inh/tau_inh                                        : 1
#''')


SYN_PROB=0.5 #0.2

def f(spikes):
    #print spikes.shape
    if (spikes.shape[0])>0:
        prob=scipy.stats.bernoulli.rvs(SYN_PROB,size=spikes.shape[0])
        spikes*=prob

 
import random
from numpy import *
importSpkTrain(Nout)

Input_ex = MultipleSpikeGeneratorGroup(SpkTrainEx, clock=simclock)
Input_inh = MultipleSpikeGeneratorGroup(SpkTrainInh, clock=simclock)

OutLayer = NeuronGroup(N=Nout,  model=eqs, threshold=Vt, reset=Vr, refractory=ref, clock=simclock) #, freeze=False, implicit=True) 

Wdefinition(Nout,ExNeur,InhNeur)

importDelays()

C_ex = Connection(Input_ex,OutLayer,'g_ex',delay=True,max_delay=delta_ij.max()*ms)
C_inh = Connection(Input_inh,OutLayer,'g_inh',delay=True,max_delay=delta_ij.max()*ms)
  
for ej,ei in ((ej,ei) for ej in range(len(ExNeur)) for ei in range(Nout)):
    C_ex.delay[ej,ei] = Quantity(delta_ij[ExNeur[ej],inputInstance-1])*ms

C_ex.connect(Input_ex,OutLayer,W_ex)

for ij,ii in ((ij,ii) for ij in range(len(InhNeur)) for ii in range(Nout)):
    C_inh.delay[ij,ii] = Quantity(delta_ij[InhNeur[ij],inputInstance-1])*ms

C_inh.connect(Input_inh,OutLayer,W_inh)

SMonitor = []
SMonitor.append(SpikeMonitor(OutLayer, function=f))

OutLayer.V = Vr+(Vt-Vr)*rand(len(OutLayer)) # uniform random initialization of the  membrane voltage V of the Output layer between Vr and Vt
#OutLayer.V = random.normal(Vr,stdVr,len(OutLayer))

# Monitors ---------------------------------------------------------------------
I_ex = SpikeMonitor(Input_ex)
I_inh = SpikeMonitor(Input_inh)
M = SpikeMonitor(OutLayer,record=True)
outVolt = StateMonitor(OutLayer,'V', record=True, clock=simclock)
outGex = StateMonitor(OutLayer,'g_ex', record=True, clock=monclock)
outGinh = StateMonitor(OutLayer,'g_inh', record=True, clock=monclock)

# run and plot -----------------------------------------------------------------
run(0.05 * second)

figure()
raster_plot(M)
show()
xlabel('time (ms)')
ylabel('neuron number')
show()

#raster_plot(I_ex)
#raster_plot(I_inh)
#show()
#
figure()
plot(outVolt.times / ms, outVolt[0] / mV)
show()

figure()
plot(outGex.times, outGex[0])
plot(outGinh.times, outGinh[0])
show()



from clock import *
from network import *
import neurongroup
from units import second
import numpy
import warnings
try:
    import pylab
except:
    warnings.warn("Couldn't import pylab.")

__all__ = ['TimedArray', 'TimedArraySetter', 'set_group_var_by_array']


class TimedArray(numpy.ndarray):
    '''
    An array where each value has an associated time.
    
    Initialisation arguments:
    
    ``arr``
        The values of the array. The first index is the time index. Any
        array shape works in principle, but only 1D/2D arrays are
        supported (other shapes may work, but may not). The idea is to,
        have the shapes (T,) or (T, N) for T the number of time steps and
        N the number of neurons.
    ``times``
        A 1D array of times whose length should be the same as the first
        dimension of ``arr``. Usually it is preferable to specify a
        clock rather than an array of times, but this doesn't work in
        the case where the time intervals are not fixed.
    ``clock``
        Specify the times corresponding to array values by a clock. The
        ``t`` attribute of the clock is the time of the first value
        in the array, and the time interval is the ``dt`` attribute of
        the clock. If neither ``times`` nor ``clock`` is specified, a
        clock will be guessed in the usual way (see :class:`Clock`).
    ``start, dt``
        Rather than specifying a clock, you can specify the start time
        and time interval explicitly. Technically, this is useful
        because it doesn't create a :class:`Clock` object which can
        lead to ambiguity about which clock is the default. If dt is
        specified and start is not, start is assumed to be 0.
        
    Note that if the clock, or start time and dt, of the array should be the
    default clock values, then you should not specify clock, start or dt (see
    Technical notes below).
    
    Arbitrary slicing of the array is supported, but the clock will only
    be preserved where the intervals can be guaranteed to be fixed, that
    is except for the case where lists or numpy arrays are used on the
    time index.

    Timed arrays can be called as if they were a function of time if the
    array times are based on a clock (but not if the array times are
    arbitrary as the look up costs would be excessive). If ``x(t)`` is called
    where ``times[i]<=t<times[i]+dt`` for some index i then ``x(t)`` will
    have the value ``x[i]``. You can also call ``x(t)`` with ``t`` a 1D array.
    If x is 1D then ``x(t)[i]=x(t[i])``, if x is 2D then ``x(t)[i]=x(t[i])[i]``.
    
    Has one method:
    
    .. method: plot(*args, **kwds)
    
        Plots the values on the y axis and times on the x axis. If the array
        is 1D then this is a single plot, if it is 2D then there will be one
        plot for each second index. 3D or greater arrays are not supported.
        The args and keywords are passed to matplotlib's plot() command. In
        the 2D case, each plot is labelled with the second index.
    
    See also :class:`TimedArraySetter`, :func:`set_group_var_by_array` and
    :class:`NeuronGroup`.
    
    **Technical notes**
    
    Note that specifying a new clock, or values of start and dt, will mean
    that if you use this :class:`TimedArray` to set the value of a
    :class:`NeuronGroup` variable, it will be updated on the schedule of this
    clock, which can (due to floating point errors) induce some timing problems.
    This rarely happens, but if an occasional inaccuracy of order dt might
    conceivably be critical for your simulation, you should use
    :class:`RegularClock` objects instead of :class:`Clock` objects.
    '''
    def __new__(subtype, arr, times=None, clock=None, start=None, dt=None):
        # All numpy.ndarray subclasses need something like this, see
        # http://www.scipy.org/Subclasses
        return numpy.array(arr, copy=False).view(subtype)

    def __array_finalize__(self, orig):
        # This is called each time a new TimedArray object is created from
        # an old one, we just copy across the clock attribs here because
        # when a new one is made from an old one, the times will be the
        # same.
        try:
            self.times = orig.times
            self.clock = orig.clock
            self._t_init = orig._t_init
            self._dt = orig._dt
        except AttributeError:
            pass
        return self

    def __init__(self, arr, times=None, clock=None, start=None, dt=None):
        # Mostly this is straightforward, the point about having
        # times and clock separate is that you don't have to limit
        # yourself to fixed time intervals, although you usually
        # will do that (and some things will rely on this, such
        # as the __call__ method).
        if start is not None or dt is not None:
            if start is None:
                start = 0 * second
            if clock is not None:
                raise ValueError('Specify start and dt or block, but not both.')
            clock = Clock(t=start, dt=dt)
        if times is not None and clock is not None:
            raise ValueError('Specify times or clock but not both.')
        if times is None and clock is None:
            clock = guess_clock(clock)
            self.guessed_clock = True
        else:
            self.guessed_clock = False
        self.clock = clock
        if clock is not None:
            self._t_init = int(clock._t / clock._dt) * clock._dt
            self._dt = clock._dt
            times = clock._t + numpy.arange(len(arr)) * clock._dt
        else:
            self._t_init = None
            self._dt = None
        self.times = times

    def __getitem__(self, item):
        # __getitem__ can deal with all sorts of indexing, we consider the
        # following types specially for an array x
        #   - x[a:b:c], if x has a clock then the slice can have a clock too
        #   - x[integer], no clock for this because it's just one time value
        #   - x[a, b, ...] with a a slice, this can have a clock based on a
        # For the remaining cases, we assume that we cannot define a clock for
        # the sliced object, e.g. if x[a,b,...] with a, b numpy arrays.

        # The values are the same as the numpy array version of __getitem__ in
        # all cases
        x = numpy.ndarray.__getitem__(self, item)
        if isinstance(item, slice):
            newtimes = self.times[item]
            if self.clock is not None and len(newtimes) > 1:
                newdt = newtimes[1] - newtimes[0]
                newclock = Clock(t=newtimes[0] * second, dt=newdt * second)
                return TimedArray(x, clock=newclock)
            else:
                return TimedArray(x, self.times[item])
        if isinstance(item, int):
            return TimedArray(x, self.times[item:item + 1])
        if isinstance(item, tuple):
            item0 = item[0]
            times = self.times[item0]
            if isinstance(item0, slice) and self.clock is not None and hasattr(times, '__len__') and len(times) > 1:
                newdt = times[1] - times[0]
                newclock = Clock(t=times[0] * second, dt=newdt * second)
                return TimedArray(x, clock=newclock)
            if not isinstance(times, numpy.ndarray):
                times = numpy.array([times])
            return TimedArray(x, times)
        times = self.times[item]
        if not isinstance(times, numpy.ndarray):
            times = numpy.array([times])
        return TimedArray(x, times)

    def __getslice__(self, start, end):
        # Just use __getitem__ for this (it's been deprecated since Python 2.0
        # but you need to implement it because the base class does)
        return self.__getitem__(slice(start, end))

    def plot(self, *args, **kwds):
        if self.size > self.times.size and len(self.shape) == 2:
            for i in xrange(self.shape[1]):
                kwds['label'] = str(i)
                self[:, i].plot(*args, **kwds)
        else:
            pylab.plot(self.times, self, *args, **kwds)

    def __call__(self, t):
        global tlast
        if self.clock is None:
            raise ValueError('Can only call timed arrays if they are based on a clock.')
        else:
            if isinstance(t, (list, tuple)):
                t = numpy.array(t)
            if isinstance(t, neurongroup.TArray):
                # In this case, we know that t = ones(N)*t so we just use the first value
                t = t[0]
            elif isinstance(t, numpy.ndarray):
                if len(self.shape) > 2:
                    raise ValueError('Calling TimedArray with array valued t only supported for 1D or 2D TimedArray.')
                if len(self.shape) == 2 and len(t) != self.shape[1]:
                    raise ValueError('Calling TimedArray with array valued t on 2D TimedArray requires len(t)=arr.shape[1]')
                #t = numpy.array((t-self._t_init)/self._dt, dtype=int)
                t = numpy.array(numpy.rint((t - self._t_init) / self._dt), dtype=int)
                t[t < 0] = 0
                t[t >= len(self.times)] = len(self.times) - 1
                if len(self.shape) == 1:
                    return numpy.asarray(self)[t]
                return numpy.asarray(self)[t, numpy.arange(len(t))]
            t = float(t)
            ot = t
            #t = int((t-self._t_init)/self._dt)
            t = int(numpy.rint((t - self._t_init) / self._dt))
#            if t-tlast!=1:
#                print 'int t:', t, tlast
#                print 't:', repr(ot)
#                print 't_init:', repr(self._t_init)
#                print 't/dt:', repr((ot-self._t_init)/self._dt)
#            tlast = t
            if t < 0: t = 0
            if t >= len(self.times): t = len(self.times) - 1
            return numpy.asarray(self)[t]

tlast = 0


class TimedArraySetter(NetworkOperation):
    '''
    Sets NeuronGroup values with a TimedArray.
    
    At the beginning of each update step, this object will set the
    values of a given state variable of a group with the value from
    the array corresponding to the current simulation time.
    
    Initialisation arguments:
    
    ``group``
        The :class:`NeuronGroup` to which the variable belongs.
    ``var``
        The name or index of the state variable in the group.
    ``arr``
        The array of values used to set the variable in the group.
        Can be an array or a :class:`TimedArray`. If it is an array,
        you should specify the ``times`` or ``clock`` arguments, or
        leave them blank to use the default clock.
    ``times``
        Times corresponding to the array values, see :class:`TimedArray`
        for more details.
    ``clock``
        The clock for the :class:`NetworkOperation`. If none is specified,
        use the group's clock. If ``arr`` is not a :class:`TimedArray`
        then this clock will be used to initialise it too.
    ``start, dt``
        Can specify these instead of a clock (see :class:`TimedArray` for
        details).
    ``when``
        The standard :class:`NetworkOperation` ``when`` keyword, although
        note that the default value is 'start'.
    '''
    def __init__(self, group, var, arr, times=None, clock=None, start=None, dt=None, when='start'):
        if clock is None:
            if isinstance(arr, TimedArray):
                self.clock = clock = arr.clock
            else:
                self.clock = clock = group.clock
        else:
            self.clock = clock
        self.when = when
        self.group = group
        self.var = var
        if not isinstance(arr, TimedArray):
            arr = TimedArray(arr, times=times, clock=clock, start=start, dt=dt)
        self.arr = arr
        self.reinit()

    def __call__(self):
        if self.arr.clock is None:
            # in this case, the time intervals need not be fixed so we
            # have to step through the array until we find the appropriate
            # one
            tcur = self.clock._t
            while True:
                if self._cur_i == len(self.arr.times) - 1:
                    self.group.state_(self.var)[:] = self.arr[self._cur_i]
                    return
                ti_now = self.arr.times[self._cur_i]
                ti_next = self.arr.times[self._cur_i + 1]
                if ti_next >= tcur:
                    self.group.state_(self.var)[:] = self.arr[self._cur_i]
                    return
                self._cur_i += 1
        else:
            self.group.state_(self.var)[:] = self.arr(self.clock._t)

    def reinit(self):
        if self.arr.clock is None:
            self._cur_i = 0

def set_group_var_by_array(group, var, arr, times=None, clock=None, start=None, dt=None):
    '''
    Sets NeuronGroup values with a TimedArray.
    
    Creates a :class:`TimedArraySetter`, see that class for details.
    '''
    array_setter = TimedArraySetter(group, var, arr, times=times, clock=clock, start=start, dt=dt)
    group._owner.contained_objects.append(array_setter)



'''
Still to do:

* Estimate with and without compiling turned on
* Investigate and estimate the cost of cache misses in
  the spiking part
'''
from brian import *
from run_opts import *
import time
import scipy
import numpy

def network_and_lazy_group(N=1):
    G = [NeuronGroup(1, model='dV/dt=-V/(10*ms):volt') for _ in range(N)]
    net = Network(G)
    net.run(0 * ms)
    #print len(net._update_schedule[id(defaultclock)])
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def network_lazy_group_and_zero_connections(N=1):
    G = NeuronGroup(1, model='dV/dt=-V/(10*ms):volt')
    C = [ Connection(G, G) for _ in range(N) ]
    net = Network(G, C)
    # problem with MultiConnection, this reproduces part of the activity of
    # network.prepare so that MultiConnection isn't used
    net.set_clock()
    net.connections = C
    for c in C:
        c.compress()
    net._build_update_schedule()
    net.prepared = True
    net.run(0 * ms)
    #print len(net._update_schedule[id(defaultclock)])
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def linear_network(N=100, vars=None):
    if vars is None:
        vars = 3
        eqs = '''
        dV/dt = (ge+gi-(V+49*mV))/(20*ms) : volt
        dge/dt = -ge/(5*ms) : volt
        dgi/dt = -gi/(10*ms) : volt
        '''
    else:
        eqs = ''
        for i in range(vars):
            # upper triangular differential equation
            eqs += 'dV' + str(i) + '/dt=-(' + '+'.join('V' + str(j) for j in range(i + 1)) + ')/(10*ms):volt\n'
    G = NeuronGroup(N, model=eqs)
    net = Network(G)
    net.run(0 * ms)
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def linear_network_cachemiss(N=100, vars=None):
    if vars is None:
        vars = 3
        eqs = '''
        dV/dt = (ge+gi-(V+49*mV))/(20*ms) : volt
        dge/dt = -ge/(5*ms) : volt
        dgi/dt = -gi/(10*ms) : volt
        '''
    else:
        eqs = ''
        for i in range(vars):
            # upper triangular differential equation
            eqs += 'dV' + str(i) + '/dt=-(' + '+'.join('V' + str(j) for j in range(i + 1)) + ')/(10*ms):volt\n'
    G = NeuronGroup(N, model=eqs)
    x = numpy.ones(cachemiss_array_size)
    @network_operation
    def hitcache():
        x[:] = 1.
    net = Network(G, hitcache)
    net.run(0 * ms)
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def spiking_network(rate, synapses_per_neuron=1):
    # rate is per neuron firing rate
    eqs = 'dV/dt=-V/(1*Msecond):volt'
    rate = int(N_spiking_network * rate * defaultclock.dt)
    G = NeuronGroup(N_spiking_network, model=eqs, threshold=10 * mV, reset=1 * volt)
    H = NeuronGroup(synapses_per_neuron, model='V:volt')
    G.V[:rate] = 1 * volt
    G.V[rate:] = 0 * volt
    C = Connection(G, H, 'V')
    C.connect_full(G, H, weight=1 * nvolt)
    net = Network(G, H, C)
    net.run(0 * ms)
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def threshold(N=100, abovethresh=50):
    G = NeuronGroup(N, model='V:volt', threshold=1 * mV)
    G.V[:abovethresh] = 2 * mV
    net = Network(G)
    net.run(0 * ms)
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def threshold_cachemiss(N=100, abovethresh=50):
    G = NeuronGroup(N, model='V:volt', threshold=1 * mV)
    G.V[:abovethresh] = 2 * mV
    x = numpy.ones(cachemiss_array_size)
    @network_operation
    def hitcache():
        x[:] = 1.
    net = Network(G, hitcache)
    net.run(0 * ms)
    reinit_default_clock()
    t = time.time()
    net.run(duration)
    return time.time() - t

def do_time(f, *params):
    return (f(*params) * second) / (duration / defaultclock.dt)

time_network_and_lazy_group = [ do_time(network_and_lazy_group, i) for i in range(1, max_groups + 1) ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(range(1, max_groups + 1), time_network_and_lazy_group)
overhead_network = intercept * second
overhead_neurongroup = slope * second
print 'Network update overhead per dt =', overhead_network
print 'NeuronGroup overhead per dt (approx, with 1 linear neuron) =', overhead_neurongroup
subplot(331)
plot(range(1, max_groups + 1), time_network_and_lazy_group)
plot(range(0, max_groups + 1), [intercept + i * slope for i in range(0, max_groups + 1)])
title('Groups overhead')

time_network_lazy_group_and_zero_connections = [ do_time(network_lazy_group_and_zero_connections, i) for i in range(max_connections + 1) ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(range(max_connections + 1), time_network_lazy_group_and_zero_connections)
overhead_connection = slope * second
print 'Connection overhead per dt =', overhead_connection
print 'Sanity check, these should be roughly equal:', intercept * second, ',', overhead_network + overhead_neurongroup
subplot(332)
plot(range(max_connections + 1), time_network_lazy_group_and_zero_connections)
plot(range(max_connections + 1), [intercept + i * slope for i in range(max_connections + 1)])
title('Connections overhead')

time_linear_network = [ do_time(linear_network, N) for N in N_linear_network ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(N_linear_network, time_linear_network)
perneuron = slope * second
print 'Sanity check, these should be roughly equal:', intercept * second, ',', overhead_network + overhead_neurongroup
print 'Time per linear neuron with numstates=3 per dt =', perneuron
subplot(333)
plot(N_linear_network, time_linear_network)
plot([0] + N_linear_network, [intercept + i * slope for i in [0] + N_linear_network])
title('Linear 3 states')
xlabel('N')

time_linear_network_cachemiss = [ do_time(linear_network_cachemiss, N) for N in N_linear_network ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(N_linear_network, time_linear_network_cachemiss)
perneuron_cachemiss = slope * second
print 'Time per linear neuron with numstates=3 per dt, cachemiss:', perneuron_cachemiss
subplot(334)
plot(N_linear_network, time_linear_network_cachemiss)
plot([0] + N_linear_network, [intercept + i * slope for i in [0] + N_linear_network])
title('Linear 3 states cachemiss')
xlabel('N')

#time_multivar_linear_network = [ do_time(linear_network, N_multivar_linear_network, M) for M in vars_linear_network ]
#fitcoeffs = polyfit(vars_linear_network, time_multivar_linear_network, 2)
#qfitcoeffs = array(fitcoeffs)*second
#perneuron_variableconst = slope*second
#print 'Time per linear neuron as function of numstates=M:', qfitcoeffs[1]/N_multivar_linear_network, 'M +', qfitcoeffs[0]/N_multivar_linear_network, 'M^2' 
#subplot(334)
#plot(vars_linear_network, time_multivar_linear_network)
#plot([0]+vars_linear_network, [fitcoeffs[2]+i*fitcoeffs[1]+i*i*fitcoeffs[0] for i in [0]+vars_linear_network])
#title('Linear in numstates, numneurons='+str(N_multivar_linear_network))

time_spiking_network = [ do_time(spiking_network, rate, fixed_syn_spiking_network) for rate in rates_spiking_network ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(rates_spiking_network, time_spiking_network)
perhertz = slope * second
perspikeandsyn = perhertz / (1.0 * Hz * fixed_syn_spiking_network * N_spiking_network * defaultclock.dt)
subplot(335)
plot(rates_spiking_network, time_spiking_network, '.')
plot([0, max(rates_spiking_network)], [intercept + float(i) * slope for i in [0, max(rates_spiking_network)]])
title('Spiking')
xlabel('rate')

time_spiking_network_syn = [ do_time(spiking_network, rate_syn, syn) for syn in syn_spiking_network ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(syn_spiking_network, time_spiking_network_syn)
persyn = slope * second / (rate_syn * N_spiking_network * defaultclock.dt)
perspike = (perspikeandsyn - persyn) * fixed_syn_spiking_network
print 'Time per synapse:', persyn
print 'Time per spike:', perspike
subplot(336)
plot(syn_spiking_network, time_spiking_network_syn, '.')
plot([0, max(syn_spiking_network)], [intercept + float(i) * slope for i in [0, max(syn_spiking_network)]])
title('Spiking')
xlabel('synapses per neuron')

time_threshold = [ do_time(threshold, N) for N in N_threshold ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(N_threshold, time_threshold)
threshold_perneuron = slope * second
print 'Thresholding operation per neuron:', threshold_perneuron
subplot(338)
plot(N_threshold, time_threshold, '.')
plot([0, max(N_threshold)], [intercept + float(i) * slope for i in [0, max(N_threshold)]])
title('Thresholding')

time_threshold_cachemiss = [ do_time(threshold_cachemiss, N) for N in N_threshold ]
slope, intercept, r, tt, stderr = scipy.stats.linregress(N_threshold, time_threshold_cachemiss)
threshold_perneuron_cachemiss = slope * second
print 'Thresholding operation per neuron, cachemiss:', threshold_perneuron_cachemiss
subplot(339)
plot(N_threshold, time_threshold_cachemiss, '.')
plot([0, max(N_threshold)], [intercept + float(i) * slope for i in [0, max(N_threshold)]])
title('Thresholding cachemiss')

show()



from brian import *
from brian.inspection import *

def test():
    '''
    Inspection module
    '''
    reinit_default_clock()
    expr = '''
x_12+=y*12 # comment
pour(water,\
   "on desk")'''
    # Check that identifiers are correctly extracted
    assert (get_identifiers(expr) == ('x_12', 'y', 'pour', 'water'))
    # Check that modified variables are correctly extracted
    assert (modified_variables(expr) == ['x_12', 'pour'])

if __name__ == '__main__':
    test()



from numpy import *
from multiprocessing.connection import Listener
from array import array

def f(x, a, n):
    s = 0
    for _ in xrange(n):
        s += sum(x) * a
    return s

address = ('localhost', 2718)     # family is deduced to be 'AF_INET'
listener = Listener(address, authkey='secret password')
print 'Listener created'

conn = listener.accept()
print 'connection accepted from', listener.last_accepted

x = conn.recv()

while True:
    item = conn.recv()
    if item is None:
        break
    a, n = item
    conn.send(f(x, a, n))

listener.close()
conn.close()



from brian import *
from brian.library.ionic_currents import *
from brian.experimental.codegen import *
import time
from scipy import weave

N = 1000
record_and_plot = N == 1

El = 10.6 * mV
EK = -12 * mV
ENa = 120 * mV
eqs = MembraneEquation(1 * uF) + leak_current(.3 * msiemens, El)
eqs += K_current_HH(36 * msiemens, EK) + Na_current_HH(120 * msiemens, ENa)
eqs += Current('I:amp')
eqs.prepare()

print eqs
print '.............................'
pycode = PythonCodeGenerator().generate(eqs, exp_euler_scheme)
print pycode
print '.............................'
ccode = CCodeGenerator().generate(eqs, exp_euler_scheme)
print ccode

neuron = NeuronGroup(N, eqs, implicit=True, freeze=True)

if record_and_plot:
    trace = StateMonitor(neuron, 'vm', record=True)

neuron.I = 10 * uA

_S_python = array(neuron._S)
_S = array(neuron._S)

ns = {'_S':_S_python, 'exp':exp, 'dt':defaultclock._dt}
pycode_comp = compile(pycode, '', 'exec')

start = time.time()
run(100 * ms)
print 'N:', N
print 'Brian:', time.time() - start

start = time.time()
hand_trace_python = []
for T in xrange(int(100 * ms / defaultclock.dt)):
    exec pycode_comp in ns
    if record_and_plot:
        hand_trace_python.append(copy(_S_python[0]))
print 'Codegen Python:', time.time() - start

start = time.time()
hand_trace_c = []
for T in xrange(int(100 * ms / defaultclock.dt)):
    dt = defaultclock._dt
    t = T * defaultclock._dt
    num_neurons = len(neuron)
    weave.inline(ccode, ['_S', 'num_neurons', 'dt', 't'],
                 compiler='gcc',
                 #type_converters=weave.converters.blitz,
                 extra_compile_args=['-O3', '-march=native', '-ffast-math'])#O2 seems to be faster than O3 here
    if record_and_plot:
        hand_trace_c.append(copy(_S[0]))
print 'Codegen C:', time.time() - start

if record_and_plot:
    plot(trace[0])
    plot(array(hand_trace_python))
    plot(array(hand_trace_c))
    show()



from brian import *
from brian.hears import *

sound = whitenoise(10*ms)

delays = linspace(-1*ms, 1*ms, 1000)

fd = FractionalDelay(sound, delays)
fd2 = FractionalDelay(sound, array(delays*sound.samplerate, dtype=int)/sound.samplerate)

I = fd.process()
I2 = fd2.process()

imshow(I, interpolation='nearest', aspect='auto')
figure()
imshow(I2, interpolation='nearest', aspect='auto')
show()



from brian import *

c = Clock(dt=1*ms, t=0.5*ms)

@network_operation(clock=c)
def f():
    print c.t
    if c.t>5*ms:
        c.t -= 0.01*ms
    
run(10*ms)

net = MagicNetwork()
net.prepare()
print net._update_schedule



from brian import *
from nose.tools import *

def test():
    """
    The :class:`Clock` object
    ~~~~~~~~~~~~~~~~~~~~~~~~~
    
    A :class:`Clock` object is initialised via::
    
        c = Clock(dt=0.1*msecond, t=0*msecond, makedefaultclock=False)
    
    In particular, the following will work and do the same thing::
    
        c = Clock()
        c = Clock(t=0*second)
        c = Clock(dt=0.1*msecond)
        c = Clock(0.1*msecond,0*second)
    
    Setting the ``makedefaultclock=True`` argument sets the newly
    created clock as the default one.

    The default clock
    ~~~~~~~~~~~~~~~~~
    
    The default clock can be found using the :func:`get_default_clock` function,
    and redefined using the :func:`define_default_clock` function, where the
    arguments passed to :func:`define_default_clock` are the same as the
    initialising arguments to the ``Clock(...)`` statement. The default
    clock can be reinitialised by calling :func:`reinit_default_clock`.
    
    A less safe way to access the default clock is to refer directly to
    the variable :data:`defaultclock`. If the default clock has been redefined, this
    won't work.

    The :func:`guess_clock` function
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    The function :func:`guess_clock` should return (in this order of priority):
    
    * The clock passed as an argument, if one was passed
    * A clock defined in the calling function, if there was one
    * The default clock otherwise
    
    If more than one clock was defined in the calling function, it should
    raise a ``TypeError`` exception.
    """
    reinit_default_clock()

    # check that 'defaultclock' default clock exists and starts at t=0
    assert defaultclock.t < 0.001 * msecond

    # check that default clock exists and starts at t = 0
    c = guess_clock()
    assert c.t < 0.001 * msecond

    # check that passing no arguments works
    c = Clock()

    # check that passing t argument works
    c = Clock(t=1 * second)
    assert c.t > 0.9 * second

    # check that passing dt argument works
    c = Clock(dt=10 * msecond)
    assert c.dt > 9 * msecond

    # check that passing t and dt arguments works
    c = Clock(t=2 * second, dt=1 * msecond)
    assert c.t > 1.9 * second
    assert 0.5 * msecond < c.dt < 2 * msecond

    # check that making this the default clock works
    assert get_global_preference('defaultclock').dt < 9 * msecond
    c = Clock(dt=10 * msecond, makedefaultclock=True)
    assert get_global_preference('defaultclock').dt > 9 * msecond

    # check that the other ways of defining a default clock work
    define_default_clock(dt=3 * msecond)
    assert 2.9 * msecond < get_global_preference('defaultclock').dt < 3.1 * msecond

    # check that the get_default_clock function works
    assert 2.9 * msecond < get_default_clock().dt < 3.1 * msecond

    # check that passing unnamed arguments in the order dt, t works
    c = Clock(10 * msecond, 1 * second)
    assert c.t > 0.9 * second
    assert c.dt > 9 * msecond

    # check that reinit() sets t=0
    c = Clock(t=1 * second)
    c.reinit()
    assert c.t < 0.001 * second

    # check that tick() works
    c = Clock(t=1 * second, dt=1 * msecond)
    for i in range(10): c.tick()
    assert c.t > 9 * msecond

    # check that reinit_default_clock works
    for i in range(10): get_default_clock().tick()
    reinit_default_clock()
    assert get_default_clock().t < 0.0001 * second

    # check that guess_clock passed a clock returns that clock
    assert 0.6 * second < guess_clock(Clock(t=0.7 * second)).t < 0.8 * second

    # check that guess_clock passed no clock returns the only clock we've defined in this function so far
    c = Clock()
    assert guess_clock() is c

    # check that if no clock is defined, guess_clock returns the default clock
    del c
    assert guess_clock() is get_default_clock()

    # check that if two or more clocks are defined, guess_clock raises a TypeError
    c = Clock()
    d = Clock()
    assert_raises(TypeError, guess_clock)
    del d

    # check that if we have a calling stack, the innermost clock is found
    c = Clock()
    def f():
        d = Clock()
        assert guess_clock() is d
        del d
        return guess_clock()
    assert f() is c

    # cleanup: reset the default clock to its default state
    set_global_preferences(defaultclock=defaultclock)



from brian import NeuronGroup, SpikeCounter, Network, ms, second, isscalar

def fun(sigma, args):
    """
    This function computes the mean firing rate of a LIF neuron with
    white noise input current (OU process with threshold).
    """
    if not isscalar(sigma):
        raise Exception('sigma must be a scalar')
    N = args['N']
    tau = args['tau']
    model = args['model']
    reset = args['reset']
    threshold = args['threshold']
    duration = args['duration']
    G = NeuronGroup(N, model=model, reset=reset, threshold=threshold)
    M = SpikeCounter(G)
    net = Network(G, M)
    net.run(duration)
    r = M.nspikes * 1.0 / N
    return r

if __name__ == '__main__':
    from playdoh import *
    import numpy as np
    import time

    N = 2000
    tau = 10 * ms
    model = '''dV/dt=-V/tau+sigma*(2/tau)**.5*xi : 1'''
    reset = 0
    threshold = 1
    duration = 1 * second

    sigmas = list(np.linspace(.3, .6, 8))

    t = time.clock()
    for s in sigmas:
        r = fun(s, dict(N=N,
                           tau=tau,
                           model=model,
                           reset=reset,
                           threshold=threshold,
                           duration=duration))
        print "Mean firing rate with sigma = %.2f : %.2f Hz" % (s, r)
    print "Serial time = %.3f s" % (time.clock() - t)
    print

    dfun = distribute(fun, dict(N=N,
                                   tau=tau,
                                   model=model,
                                   reset=reset,
                                   threshold=threshold,
                                   duration=duration))

    t = time.clock()
    results = dfun(sigmas)
    t = time.clock() - t

    for s, r in zip(sigmas, results):
        print "Mean firing rate with sigma = %.2f : %.2f Hz" % (s, r)
    print "Parallel time = %.3f s" % t




"""
Here I try do define multiple events on a NeuronGroup (not just spikes).

How to make this in a nice way?
1) Insert it in NeuronGroup. We would pass a list of threshold/reset. It is
   handled as currently.
2) Have multiple states, as we thought for refractoriness. But instead of
   having one system of equations for each state, we just have a separate
   threshold and reset.
3a) Insert it in NeuronGroup, but have a new integer variable, "state". Each
   state corresponds to a specific additional event type. Additional threshods/
   resets are called only when the neuron is in state 0, 1, etc. Ex:
     threshold=("v>1","v>0.8","v<0.8") # on state: all, 0, 1
     reset=("v=0","state=1","state=0")
 b) This could also be implicit, that is, we assume that event triggering is
   prevented as long as the condition still holds true. This would give simply:
     threshold=("v>1","v>0.8")
     reset="v=0"
   and the state would be automatically dealt with, possibly via a new threshold
   class, e.g. PositiveCrossing("v>1").
4) Have a new class attached to a group, e.g.:
   ParasiteGroup(P,threshold,reset)
   Threshold for that class could, by default, be a positive crossing.
   This could create a subgroup of P.
   
I prefer 3 & 4. Option 4 is simpler, but perhaps might integrate less
nicely with STDP (but not sure). But it also works nicely with SpikeMonitor.

Here option 4 is implemented.
"""
from brian import *
from brian.utils.circular import *
import copy

class PositiveCrossing(Threshold):
    '''
    The event (spike) is triggered only the first time
    the threshold condition is satisfied. That is, after an event is triggered,
    further events are prevented until the condition is false again.

    **Initialised as:** ::
    
        PositiveCrossing(threshold)

    where ``threshold`` is another threshold.
    
    Warning: an instance can be used with only one group.
    '''
    # Not the most efficient way to do it, but it works!
    def __init__(self,threshold):
        self._threshold=threshold
        self._firsttime=True

    def __call__(self, P):
        if self._firsttime:
            self._firsttime=False
            self._available=ones(len(P),dtype=bool_)
        all_spikes=self._threshold(P)
        spikes=all_spikes[self._available[all_spikes]]
        self._available[:]=True
        self._available[all_spikes]=False
        return spikes

    def __repr__(self):
        return "Positive crossing threshold"
    
def ParasiteGroup(P, threshold, reset=NoReset()):
    '''
    This is a NeuronGroup that tracks events on P, that is,
    that produces a spike when the threshold condition is true and was
    false at the previous time step (positive crossing).
    
    Optionally, a reset operation can be applied (maybe we could
    remove this).
    
    Events are stored for as long as spikes in P (could be an option).
    '''
    # This would be probably nicer in a proper subclass
    # or WatcherGroup?
    Q=P[:] # New subgroup
    Q._owner=Q # This separates it from its parents (otherwise it is not run)
    Q.set_instance_id(level=1) # This makes available to magic tools
    Q._state_updater=LazyStateUpdater() # Does not update the state matrix
    Q._threshold=PositiveCrossing(StringThreshold(threshold, level=1)) # Defines a new threshold
    Q._resetfun=reset # No reset
    Q.LS = SpikeContainer(Q._max_delay,
                          useweave=get_global_preference('useweave'),
                          compiler=get_global_preference('weavecompiler')) # Spike storage
    return Q
    
if __name__=='__main__':
    tau=10*ms
    P=NeuronGroup(10,model='dv/dt=(1.1-v)/tau:1',threshold=1,reset=0)
    P2=ParasiteGroup(P,"v>0.6")
    P3=ParasiteGroup(P,"v>0.8")
    
    M=StateMonitor(P,'v',record=0)
    S=SpikeMonitor(P)
    S2=SpikeMonitor(P2)
    S3=SpikeMonitor(P3)
    run(100*ms)
    subplot(411)
    plot(M.times/ms,M[0])
    subplot(412)
    raster_plot(S)
    xlim(0,100)
    subplot(413)
    raster_plot(S2)
    xlim(0,100)
    subplot(414)
    raster_plot(S3)
    xlim(0,100)
    show()
    


'''
Seems to make it go slower rather than faster...
'''

from brian import *
import numpy
from scipy import weave

neuron_column_width = 512 # various values, none of them make it faster

class CacheConnection(Connection):
    def propagate(self, spikes):
        if not self.iscompressed:
            self.compress()
        if len(spikes):
            if not isinstance(spikes, numpy.ndarray):
                spikes = array(spikes, dtype=int)
            sv = self.target._S[self.nstate]
            rows = self.W.get_rows(spikes)
            nspikes = len(spikes)
            N = len(sv)
            code = """
                    using namespace std;
                    //vector< blitz::Array<double,1> > rowset;
                    vector< double* > rowset;
                    rowset.reserve(nspikes);
                    for(int j=0;j<nspikes;j++)
                    {
                        PyObject* _rowsj = rows[j];
                        PyArrayObject* _row = convert_to_numpy(_rowsj, "row");
                        conversion_numpy_check_type(_row, PyArray_DOUBLE, "row");
                        conversion_numpy_check_size(_row, 1, "row");
                        //blitz::Array<double,1> row = convert_to_blitz<double,1>(_row,"row");
                        //rowset[j] = row;
                        rowset[j] = (double*)_row->data;
                        Py_DECREF(_rowsj);                        
                    }
                    for(int start=0; start<N; start+=neuron_column_width)
                    {
                        int end = start+neuron_column_width;
                        if(end>N) end=N;
                        for(int j=0;j<nspikes;j++)
                        {
                            /*PyObject* _rowsj = rows[j];
                            PyArrayObject* _row = convert_to_numpy(_rowsj, "row");
                            conversion_numpy_check_type(_row, PyArray_DOUBLE, "row");
                            conversion_numpy_check_size(_row, 1, "row");
                            blitz::Array<double,1> row = convert_to_blitz<double,1>(_row,"row");*/
                            //blitz::Array<double,1> row = rowset[j];
                            double *row = rowset[j];
                            for(int k=start;k<end;k++)
                                //sv(k) += row(k);
                                sv(k) += row[k];
                            //Py_DECREF(_rowsj);
                        }
                    }
                    """
            weave.inline(code, ['sv', 'spikes', 'nspikes', 'N', 'rows', 'neuron_column_width'],
                         compiler=self._cpp_compiler,
                         type_converters=weave.converters.blitz,
                         extra_compile_args=self._extra_compile_args,
                         headers=['<vector>'])

if __name__=='__main__':

    log_level_debug()

    import time
    
    #set_global_preferences(usenewpropagate=False)
    
    N = 10000
    tau = 10*ms
    duration = 100*ms
    p = 0.1
    
    eqs = '''
    dv/dt = -v/(10*ms) : 1
    '''
    
    G = NeuronGroup(N, eqs, reset=-2, threshold=-1)
    G.v = rand(N)-2
    
    C = Connection(G, G, 'v', weight=1e-10, sparseness=p, structure='dense')
    #C = CacheConnection(G, G, 'v', weight=1e-10, sparseness=p, structure='dense')
    
    M = SpikeMonitor(G, record=False)
    
    run(1*ms)

    forget(C)
    start = time.time()
    run(duration)
    end = time.time()
    t_noconnection = end-start
    recall(C)    
    
    start = time.time()
    run(duration)
    end = time.time()
    t_both = end-start
    t_connection = t_both - t_noconnection
    
    print 'Propagation time:', t_connection
    print 'Average neuron firing rate:', M.nspikes/duration/N
    print 'Average spikes per time step:', M.nspikes/(duration/defaultclock.dt)
    print 'Average synapses per time step:', M.nspikes*(N*p)/(duration/defaultclock.dt)
    
#    raster_plot(M)
#    show()



from brian import *
from nose.tools import *
import numpy

def test():
    """
    Each of the following functions f(x) should use units if they are passed a
    :class:`Quantity` object or fall back on their numpy versions
    otherwise.
    
        sqrt, log, exp, sin, cos, tan, arcsin, arccos, arctan,
        sinh, cosh, tanh, arcsinh, arccosh, arctanh    
    """
    reinit_default_clock()
    # check sqrt behaves as expected
    x = 3 * second
    z = numpy.array([3., 2.])
    assert (have_same_dimensions(sqrt(x), second ** 0.5))
    assert (isinstance(sqrt(z), numpy.ndarray))

    # check the return types are right for all other functions
    x = 0.5 * second / second
    funcs = [
        sqrt, log, exp, sin, cos, tan, arcsin, arccos, arctan,
        sinh, cosh, tanh, arcsinh, arccosh, arctanh
            ]
    for f in funcs:
        assert (isinstance(f(x), Quantity))
        assert (isinstance(f(z), numpy.ndarray))

    # check that attempting to use these functions on something with units fails
    funcs = [
        log, exp, sin, cos, tan, arcsin, arccos, arctan,
        sinh, cosh, tanh, arcsinh, arccosh, arctanh
            ]
    x = 3 * second
    for f in funcs:
        assert_raises(DimensionMismatchError, f, x)

if __name__ == '__main__':
    test()



from brian import *
set_global_preferences(useweave=True)
import cuba_runopts
cuba_runopts.duration = 100 * ms
import pycallgraph
from cuba import *

cg_func = 'Connection.do_propagate'

def ff(pat):
    def f(call_stack, module_name, class_name, func_name, full_name):
        if not 'brian' in module_name: return False
        for n in call_stack + [full_name]:
            if pat in n:
                return True
        return False
    return f

def cuba(P, Pe, Pi, Ce, Ci, M, net):
    net.run(duration)

c = cubanetwork(4000)
pycallgraph.start_trace(filter_func=ff(cg_func))
cuba(*c)
pycallgraph.stop_trace()
pycallgraph.make_dot_graph('callgraphs/cuba-callgraph-' + cg_func + '.png')



'''
Bridge experiment (current-clamp)
'''
from brian import *
from brian.library.electrophysiology import *

defaultclock.dt = .01 * ms

#log_level_debug()

taum = 20 * ms
gl = 20 * nS
Cm = taum * gl
Re = 50 * Mohm
Ce = 0.5 * ms / Re
N = 10

eqs = Equations('''
dvm/dt=(-gl*vm+i_inj)/Cm : volt
#Rbridge:ohm
CC:farad
I:amp
''')
eqs += electrode(.6 * Re, Ce)
#eqs+=current_clamp(vm='v_el',i_inj='i_cmd',i_cmd='I',Re=.4*Re,Ce=Ce,
#                   bridge='Rbridge')
eqs += current_clamp(vm='v_el', i_inj='i_cmd', i_cmd='I', Re=.4 * Re, Ce=Ce,
                   bridge=Re, capa_comp='CC')
setup = NeuronGroup(N, model=eqs)
setup.I = 0 * nA
setup.v = 0 * mV
#setup.Rbridge=linspace(0*Mohm,60*Mohm,N)
setup.CC = linspace(0 * Ce, Ce, N)
recording = StateMonitor(setup, 'v_rec', record=True)

run(50 * ms)
setup.I = .5 * nA
run(200 * ms)
setup.I = 0 * nA
run(150 * ms)
for i in range(N):
    plot(recording.times / ms + i * 400, recording[i] / mV, 'k')
show()



import re, inspect

# We just use this class to identify certain variables
class Symbol(str): pass

# This class is basically just a mutable string
class OutputCode(object):
    def __init__(self):
        self.code = ''

    def __iadd__(self, code):
        self.code = self.code + code
        return self

# Iterating over instances of this class generates code
# for iterating over a C++ array, it yields a single
# Symbol object, the variable name of the value in the
# array
class Array(object):
    def __init__(self, code, name, dtype='double'):
        self.name = name
        self.dtype = dtype
        self.code = code

    def __iter__(self):
        def f():
            self.code += 'for(int {name}_index=0; {name}_index<{name}_len; {name}_index++){{\n'.format(name=self.name)
            self.code += '{dtype} &{name} = {name}_array[{name}_index];\n'.format(dtype=self.dtype, name=self.name)
            yield Symbol(self.name)
            self.code += '}\n'
        return f()

# Instances of this class generate C++ code from Python syntax
# code snippets, replacing variable names that are a Symbol in the
# namespace with the value of that Symbol.
class Evaluator(object):
    def __init__(self, code):
        self.code = code

    def __call__(self, code):
        # The set of variables in the code snippet
        vars = re.findall(r'\b(\w+)\b', code)
        # Extract any names from the namespace of the calling frame
        frame = inspect.stack()[1][0]
        globals, locals = frame.f_globals, frame.f_locals
        values = {}
        for var in vars:
            if var in locals:
                values[var] = locals[var]
            elif var in globals:
                values[var] = globals[var]
        # Replace any variables whose values are Symbols with their values
        for var, value in values.iteritems():
            if isinstance(value, Symbol):
                code = re.sub(r'\b{var}\b'.format(var=var), str(value), code)
        # Turn Python snippets into C++ (just a simplified version for now)
        code = code.replace(';', '\n')
        lines = [line.strip() for line in code.split('\n')]
        code = ''.join(line + ';\n' for line in lines)
        self.code += code

if __name__ == '__main__':
    code = OutputCode()
    evaluate = Evaluator(code)
    X = Array(code, 'values')
    for x in X:
        evaluate('x += 5; x *= 2')
    print code.code



from brian import *
from brian.utils.approximatecomparisons import *

from nose.tools import *

def test():
    """
    Spike containers
    ~~~~~~~~~~~~~~~~
    
    A spike container is either an iterable object or a callable object which
    returns an iterable object. For example, a list is a spike container, as
    is a generator, as is a function which returns a list, or a generator
    function.
    
    :class:`MultipleSpikeGeneratorGroup`
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Called as::
    
        MultipleSpikeGeneratorGroup(spiketimes[,clock])
        
    spiketimes is a list of spike containers, one for each neuron in the group.
    The elements of the spike containers are spike times. Callable spike
    containers are called when the group is reinitialised. If you provide
    a generator rather than a callable object, reinitialising the group will
    not reinitialise the generator. If the containers are numpy arrays units
    will not be checked (times should be in seconds).
    
    So for example, the following will correspond to a group of 2 neurons, where
    the first fires at times 0ms, 2ms and 5ms, and the second fires at times
    1ms and 3ms::
    
        spiketimes = [[0*msecond, 2*msecond, 5*msecond],
                      [1*msecond, 3*msecond]]
        G = MultipleSpikeGeneratorGroup(spiketimes)
    
    You could do the same thing with generator functions (rather perversely in
    this case)::
    
        def st1():
            yield 0*msecond
            yield 2*msecond
            yield 5*msecond
        def st2():
            yield 1*msecond
            yield 3*msecond
        G = MultipleSpikeGeneratorGroup([st1(), st2()])

    Note that if two or more spike times fall within the same dt, spikes will stack up
    and come out one per dt until the stack is exhausted. A warning will be generated
    if this happens.
    
    If a clock is provided, updates of the group will be synchronised with
    that clock, otherwise the standard clock guessing procedure will be used
    (see :func:`~brian.clock.guess_clock` in the :mod:`~brian.clock` module).
    
    :class:`SpikeGeneratorGroup`
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Called as::
    
        SpikeGeneratorGroup(N,spiketimes[,clock])
    
    where N is the number of neurons in the group, and spiketimes is a spike
    container, whose elements are tuples (i,t) meaning neuron i fires at
    time t. Pairs (i,t) need to be sorted in time unless spiketimes is a tuple
    or list. For example::
    
        from math import random
        def spikefirer(N,lower,upper):
            nexttime = random.uniform(lower,upper)
            while True:
                yield (random.randint(0,N-1),nexttime)
                nexttime = nexttime + random.uniform(lower,upper)
        G = SpikeGeneratorGroup(10,uniform_isi(10,0*msecond,10*msecond))
    
    would give a neuron group P with 10 neurons, where a random one of the neurons fires 
    with an interval between spikes which is uniform in (0ms, 10ms). If spiketimes is
    callable, it will be called when the group is reinitialised. If you provide
    a generator rather than a callable object, reinitialising the group will
    not reinitialise the generator.
    
    Note that if a neuron fires more than one spike in a given interval dt, additional
    spikes will be discarded.
    
    If a clock is provided, updates of the group will be synchronised with
    that clock, otherwise the standard clock guessing procedure will be used
    (see :func:`~brian.clock.guess_clock` in the :mod:`~brian.clock` module).
    
    :class:`PulsePacket`
    ~~~~~~~~~~~~~~~~~~~~
    
    Fires a Gaussian distributed packet of n spikes with given spread, called as::
    
        PulsePacket(t,n,sigma[,clock])
        
    You can change the parameters by calling the method ``generate(t,n,sigma)``.
    
    If a clock is provided, updates of the group will be synchronised with
    that clock, otherwise the standard clock guessing procedure will be used
    (see :func:`~brian.clock.guess_clock` in the :mod:`~brian.clock` module).
    """
    reinit_default_clock()

    # helper function for running a simple network
    def mininet(grouptype, *args, **kwds):
        reinit_default_clock()
        G = grouptype(*args, **kwds)
        M = SpikeMonitor(G, True)
        net = Network(G, M)
        net.run(5 * msecond)
        reinit_default_clock()
        return M.spikes

    # run mininet twice to see if it handles reinit() correctly
    def mininet2(grouptype, *args, **kwds):
        reinit_default_clock()
        G = grouptype(*args, **kwds)
        M = SpikeMonitor(G, True)
        net = Network(G, M)
        net.run(5 * msecond)
        spikes1 = M.spikes
        net.reinit()
        net.run(5 * msecond)
        spikes2 = M.spikes
        reinit_default_clock()
        return (spikes1, spikes2)

    # check multiple spike generator group with lists
    spiketimes = [[0 * msecond, 2 * msecond, 4 * msecond],
                  [1 * msecond, 3 * msecond]]
    spikes = mininet(MultipleSpikeGeneratorGroup, spiketimes)

    def test1(spikes):
        assert len(spikes) == 5
        i, t = zip(*spikes) # zip(*...) is the inverse of zip, so i is the ordered list of neurons that fired, and t is the ordered list of times
        assert i == (0, 1, 0, 1, 0) # check that the order of neuron firings is correct
        for s1, s2 in enumerate(t):
            assert is_approx_equal(s1 * msecond, s2) # the firing times are (0,1,2,3,4)ms
    test1(spikes)

    # check multiple spike generator group with arrays
    # NOTE: Units are not checked, array has to be in seconds
    spiketimes = [array([0.0, 0.002, 0.004]),
                  array([0.001, 0.003])]
    spikes = mininet(MultipleSpikeGeneratorGroup, spiketimes)
    test1(spikes)
    
    # multiple spike generator group with generator and period
    def gen1():
        yield 0 * msecond
    def gen2():
        yield 1 * msecond
    spikes = mininet(MultipleSpikeGeneratorGroup, [gen1, gen2], period=2*ms)
    test1(spikes)

    # check that given a different clock it works as expected, wrap in a function to stop magic functions from
    # picking up the clock objects we define here
    def testwithclock():
        spikes = mininet(MultipleSpikeGeneratorGroup, spiketimes, clock=Clock(dt=0.1 * msecond))
        test1(spikes)
        spikes = mininet(MultipleSpikeGeneratorGroup, spiketimes, clock=Clock(dt=2 * msecond))
        assert len(spikes) == 5
        i, t = zip(*spikes) # zip(*...) is the inverse of zip, so i is the ordered list of neurons that fired, and t is the ordered list of times
        for s1, s2 in zip([0, 2, 2, 4, 4], t):
            assert is_approx_equal(s1 * msecond, s2) # the firing times are (0,2,2,4,4)ms
    testwithclock()

    # check multiple spike generator group with generators
    def st1():
        yield 0 * msecond
        yield 2 * msecond
        yield 4 * msecond

    def st2():
        yield 1 * msecond
        yield 3 * msecond
    spikes = mininet(MultipleSpikeGeneratorGroup, [st1(), st2()])
    test1(spikes)

    # check reinit
    spikes1, spikes2 = mininet2(MultipleSpikeGeneratorGroup, [st1, st2])
    test1(spikes1)
    test1(spikes2)

    # spike generator with list
    spiketimes = [(0, 0 * msecond), (1, 1 * msecond), (0, 2 * msecond), (1, 3 * msecond), (0, 4 * msecond) ]
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)
    test1(spikes)

    # spike generator with list (already sorted so pass sort=False)
    spiketimes = [(0, 0 * msecond), (1, 1 * msecond), (0, 2 * msecond), (1, 3 * msecond), (0, 4 * msecond) ]
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes, sort=False)
    test1(spikes)

    # spike generator with unsorted (inversely sorted) list (sort=True is default)
    spiketimes = [(0, 4 * msecond), (1, 3 * msecond), (0, 2 * msecond), (1, 1 * msecond), (0, 0 * msecond) ]
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)
    test1(spikes)

    # check that it works with a clock
    def testwithclock():
        spikes = mininet(SpikeGeneratorGroup, 2, spiketimes, clock=Clock(dt=0.1 * msecond))
        test1(spikes)
        spikes = mininet(SpikeGeneratorGroup, 2, spiketimes, clock=Clock(dt=2 * msecond))
        assert len(spikes) == 5
        i, t = zip(*spikes) # zip(*...) is the inverse of zip, so i is the ordered list of neurons that fired, and t is the ordered list of times
        for s1, s2 in zip([0, 2, 2, 4, 4], t):
            assert is_approx_equal(s1 * msecond, s2) # the firing times are (0,2,2,4,4)ms
    testwithclock()

    # spike generator with a function returning a list
    def return_spikes():
        return [(0, 0 * msecond), (1, 1 * msecond), (0, 2 * msecond),
                (1, 3 * msecond), (0, 4 * msecond) ]
        
    spikes = mininet(SpikeGeneratorGroup, 2, return_spikes)
    test1(spikes)    

    # spike generator with a list of spikes with simultaneous spikes across neurons
    spiketimes = [(0, 0 * msecond), (1, 0 * msecond), (0, 2 * msecond),
                (1, 2 * msecond), (0, 4 * msecond), (1, 4 * msecond) ]
    def test2(spikes):            
        assert len(spikes) == 6
        #check both neurons spiked at the correct times
        for neuron in [0, 1]:
            for s1, s2 in zip([0, 2, 4], [t for i, t in spikes if i==neuron]):
                assert is_approx_equal(s1 * msecond, s2) 
    
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)
    test2(spikes)    

    # same but using the gather=True option
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes, gather=True)
    test2(spikes)    

    # same but with index arrays instead of single neuron indices
    spiketimes = [([0, 1], 0 * msecond), ([0, 1], 2 * msecond),
                  ([0, 1], 4 * msecond)]
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)
    test2(spikes)    

    # spike generator with single indices and index arrays of varying length 
    spiketimes = [([0, 1], 0 * msecond), (0, 1 * msecond), (1, 2 * msecond), ([0], 3 * msecond) ]
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)    
    def test3(spikes):            
        assert len(spikes) == 5
        #check both neurons spiked at the correct times
        for s1, s2 in zip([0, 1, 3], [t for i, t in spikes if i==0]):
            assert is_approx_equal(s1 * msecond, s2)
        for s1, s2 in zip([0, 2], [t for i, t in spikes if i==1]):
            assert is_approx_equal(s1 * msecond, s2)            
    test3(spikes)

    # spike generator with an array of (non-simultaneous) spikes
    # NOTE: For an array, the times have to be in seconds and sorted
    spiketimes = array([[0, 0.0], [1, 0.001], [0, 0.002], [1, 0.003], [0, 0.004]])
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)
    test1(spikes)

    # spike generator with an array of (simultaneous) spikes
    spiketimes = array([[0, 0.0], [1, 0.0], [0, 0.002], [1, 0.002], [0, 0.004], [1, 0.004]])
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes)
    test2(spikes)

    # spike generator with an array of (simultaneous) spikes, using gather=True
    spiketimes = array([[0, 0.0], [1, 0.0], [0, 0.002], [1, 0.002], [0, 0.004], [1, 0.004]])
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes, gather=True)
    test2(spikes)
    
    # test the handling of an empty initialization and direct setting of spiketimes
    def test_attribute_setting():
        reinit_default_clock()        
        G = SpikeGeneratorGroup(2, [])
        M = SpikeMonitor(G, True)
        net = Network(G, M)
        net.run(5 * msecond)
        assert len(M.spikes) == 0
        reinit_default_clock()  
        net.reinit()
        G.spiketimes = [(0, 0 * msecond), (1, 1 * msecond), (0, 2 * msecond),
                        (1, 3 * msecond), (0, 4 * msecond)]
        net.run(5 * msecond)
        test1(M.spikes)
    test_attribute_setting()
    
    # tests a subtle difficulty when setting spiketimes and using a subgroup
    def test_attribute_setting_subgroup():
        reinit_default_clock()
        G = SpikeGeneratorGroup(2, [])
        subG = G.subgroup(2)
        M = SpikeMonitor(subG, True)
        G.spiketimes = [(0, 0 * msecond), (1, 1 * msecond), (0, 2 * msecond),
                        (1, 3 * msecond), (0, 4 * msecond)]
        G.spiketimes = [(0, 0 * msecond), (1, 1 * msecond), (0, 2 * msecond),
                        (1, 3 * msecond), (0, 4 * msecond)]
        net = Network(G, M)
        net.run(5 * msecond)
        test1(M.spikes)    
    test_attribute_setting_subgroup()
    
    # spike generator with generator
    def sg():
        yield (0, 0 * msecond)
        yield (1, 1 * msecond)
        yield (0, 2 * msecond)
        yield (1, 3 * msecond)
        yield (0, 4 * msecond)
    spikes = mininet(SpikeGeneratorGroup, 2, sg())
    test1(spikes)

    # spike generator reinit
    spikes1, spikes2 = mininet2(SpikeGeneratorGroup, 2, sg)
    test1(spikes1)
    test1(spikes2)

    # spike generator group with generator and period
    def gen():
        yield (0, 0 * msecond)
        yield (1, 1 * msecond)
    spikes = mininet(SpikeGeneratorGroup, 2, gen, period=2*ms)
    test1(spikes)

    # spike generator group with list and period
    spiketimes = [(0, 0 * msecond), (1, 1 * msecond)]
    spikes = mininet(SpikeGeneratorGroup, 2, spiketimes, period=2*ms)
    test1(spikes)

    # pulse packet with 0 spread
    spikes = mininet(PulsePacket, 2.5 * msecond, 10, 0 * msecond)
    assert len(spikes) == 10
    i, t = zip(*spikes)
    for s in t:
        assert is_approx_equal(2.5 * msecond, s)
    # do not attempt to verify the behaviour of PulsePacket here, this is
    # an interface test only

def test_poissoninput():
    eqs = Equations("dv/dt=(1-v)/(1*second) : 1")
    group = NeuronGroup(N=1, model=eqs, reset=0, threshold=1)
    input = PoissonInput(group, N = 10, rate=50 * Hz, weight = .11, state='v')
    m = SpikeCounter(group)
    net = Network(group, input, m)
    net.run(500 * ms)
    #only checks that there some spikes
    assert (m.nspikes >= 1)

if __name__ == '__main__':
    test()
    test_poissoninput()



'''
The code below uses the GPU to compute a matrix-matrix product.

Notes:

- The C code is adapted from the simpleCUBLAS sample in the CUDA SDK
- Matrices have to have Fortran ordering [is this equivalent to
  transposing? if so this is not a problem because BLAS includes
  the option to operate on transposed matrices]
- Timing comparisons are for the time required to initialise CUDA,
  upload, perform the operation, and then download again, so for
  small times the CPU performs better. On mine at n=1000 it's about
  the same, but for N=4000 the GPU is 10x faster.
- I'm not sure how you're supposed to indicate failure and return
  in a Weave inline function, so that's just ignored for the moment,
  if one of the functions returns an error the program will
  probably just crash.
'''

from numpy import *
from scipy import weave
from scipy import randn
import time

n = 4000
x = array(randn(n, n), order='F')
y = array(randn(n, n), order='F')
z = array(zeros((n, n)), order='F')

code = '''
cublasStatus status;
double *d_x = 0;
double *d_y = 0;
double *d_z = 0;
double alpha = 1.0;
double beta = 0.0;
int n2 = n*n; 

//fprintf (stderr, "fprintf working\\n");

status = cublasInit();
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! CUBLAS initialization error\\n");
    //return EXIT_FAILURE;
}

//////////////////// Alloc device mem ///////////////////////////
status = cublasAlloc(n2, sizeof(d_x[0]), (void**)&d_x);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! device memory allocation error (x)]\\n");
    //return EXIT_FAILURE;
}
status = cublasAlloc(n2, sizeof(d_y[0]), (void**)&d_y);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! device memory allocation error (y)]\\n");
    //return EXIT_FAILURE;
}
status = cublasAlloc(n2, sizeof(d_z[0]), (void**)&d_z);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! device memory allocation error (z)]\\n");
    //return EXIT_FAILURE;
}

//////////////////// Upload ///////////////////////////
status = cublasSetVector(n2, sizeof(x[0]), x, 1, d_x, 1);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! device access error (write x)\\n");
    //return EXIT_FAILURE;
}
status = cublasSetVector(n2, sizeof(y[0]), y, 1, d_y, 1);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! device access error (write y)\\n");
    //return EXIT_FAILURE;
}

//////////////////// Multiply ///////////////////////////
cublasDgemm('n', 'n', n, n, n, alpha, d_x, n, d_y, n, beta, d_z, n);
status = cublasGetError();
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! kernel execution error.\\n");
    //return EXIT_FAILURE;
}

//////////////////// Download ///////////////////////////
status = cublasGetVector(n2, sizeof(z[0]), d_z, 1, z, 1);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! device access error (read z)\\n");
    //return EXIT_FAILURE;
}

//////////////////// Free memory ///////////////////////////
status = cublasFree(d_x);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! memory free error (x)\\n");
    //return EXIT_FAILURE;
}
status = cublasFree(d_y);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! memory free error (y)\\n");
    //return EXIT_FAILURE;
}
status = cublasFree(d_z);
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! memory free error (z)\\n");
    //return EXIT_FAILURE;
}

/* Shutdown */
status = cublasShutdown();
if (status != CUBLAS_STATUS_SUCCESS) {
    fprintf (stderr, "!!!! shutdown error \\n");
    //return EXIT_FAILURE; 
}

'''

start = time.time()
weave.inline(code, ['x', 'y', 'z', 'n'],
             compiler='gcc', #msvc works too
             headers=['"cublas.h"'],
             include_dirs=['C:\\CUDA\\include'],
             libraries=['cublas'],
             library_dirs=['C:\\CUDA\\lib'],
             )
end = time.time()

print 'GPU time:', end - start

start = time.time()
w = dot(x, y)
end = time.time()

print 'CPU time:', end - start

print 'Max abs difference in values:', amax(abs(w - z))

print 'OK'



'''
Tutorial 1d: Introducing randomness
***********************************

In the previous part of the tutorial, all the neurons start
at the same values and proceed deterministically, so they all
spike at exactly the same times. In this part, we introduce
some randomness by initialising all the membrane potentials
to uniform random values between the reset and threshold
values.

We start as before:
'''
from brian import *

tau = 20 * msecond        # membrane time constant
Vt = -50 * mvolt          # spike threshold
Vr = -60 * mvolt          # reset value
El = -49 * mvolt          # resting potential (same as the reset)

G = NeuronGroup(N=40, model='dV/dt = -(V-El)/tau : volt',
              threshold=Vt, reset=Vr)

M = SpikeMonitor(G)
'''
But before we run the simulation, we set the values of the
membrane potentials directly. The notation ``G.V`` refers
to the array of values for the variable ``V`` in group ``G``. In
our case, this is an array of length 40. We set its values
by generating an array of random numbers using Brian's
``rand`` function. The syntax is ``rand(size)`` generates an
array of length ``size`` consisting of uniformly distributed
random numbers in the interval 0, 1.
'''
G.V = Vr + rand(40) * (Vt - Vr)
'''
And now we run the simulation as before.
'''
run(1 * second)

print M.nspikes
'''
But this time we get a varying number of spikes each time
we run it, roughly between 800 and 850 spikes. In the
next part of this tutorial, we introduce a bit more
interest into this network by connecting the neurons together.
'''



'''
Integrate-and-Fire models.
'''

from brian.units import *
from brian.stdunits import *
from brian.membrane_equations import *
from numpy import exp

__all__ = ['leaky_IF', 'perfect_IF', 'exp_IF', 'quadratic_IF', 'Brette_Gerstner', 'Izhikevich', \
         'AdaptiveReset', 'aEIF']

__credits__ = dict(author='Romain Brette (brette@di.ens.fr)',
                 date='April 2008')

#TODO: specific integration methods

"""
******************************************
One-dimensional integrate-and-fire models
******************************************
"""
@check_units(tau=second, El=volt)
def leaky_IF(tau, El):
    '''
    A leaky integrate-and-fire model (membrane equation).
    tau dvm/dt = EL - vm
    '''
    return MembraneEquation(tau) + \
           Current('Im=El-vm:volt', current_name='Im', El=El)

@check_units(tau=second)
def perfect_IF(tau):
    '''
    A perfect integrator.
    tau dvm/dt = ...
    '''
    return MembraneEquation(tau)

@check_units(C=farad, gL=siemens, EL=volt, VT=volt, DeltaT=volt)
def exp_IF(C, gL, EL, VT, DeltaT):
    '''
    An exponential integrate-and-fire model (membrane equation).
    '''
    return MembraneEquation(C) + \
           Current('Im=gL*(EL-vm)+gL*DeltaT*exp((vm-VT)/DeltaT):amp', \
                   gL=gL, EL=EL, DeltaT=DeltaT, exp=exp, VT=VT)

@check_units(C=farad, a=siemens / volt, EL=volt, VT=volt)
def quadratic_IF(C, a, EL, VT):
    '''
    Quadratic integrate-and-fire model.
    C*dvm/dt=a*(vm-EL)*(vm-VT)
    '''
    return MembraneEquation(C) + \
           Current('Im=a*(vm-EL)*(vm-VT):amp', \
                   a=a, EL=EL, exp=exp, VT=VT)

"""
******************************************
Two-dimensional integrate-and-fire models
******************************************
"""
@check_units(a=1 / second, b=1 / second)
def Izhikevich(a=0.02 / ms, b=0.2 / ms):
    '''
    Returns a membrane equation for the Izhikevich model
    (variables: vm and w).
    '''
    return MembraneEquation(1.) + \
           Current('''
           Im=(0.04/ms/mV)*vm**2+(5/ms)*vm+140*mV/ms-w : volt/second
           dw/dt=a*(b*vm-w)                            : volt/second
           ''', current_name='Im')

@check_units(C=farad, gL=siemens, EL=volt, VT=volt, DeltaT=volt, \
             tauw=second, a=siemens)
def Brette_Gerstner(C=281 * pF, gL=30 * nS, EL= -70.6 * mV, VT= -50.4 * mV, \
                   DeltaT=2 * mV, tauw=144 * ms, a=4 * nS):
    '''
    Returns a membrane equation for the Brette-Gerstner model.
    Default: a regular spiking cortical cell.

    Brette, R. and W. Gerstner (2005).
    Adaptive exponential integrate-and-fire model as an effective
    description of neuronal activity.
    Journal of Neurophysiology 94: 3637-3642.
    '''
    return exp_IF(C, gL, EL, VT, DeltaT) + \
           IonicCurrent('dw/dt=(a*(vm-EL)-w)/tauw:amp', \
                        a=a, EL=EL, tauw=tauw)

aEIF = Brette_Gerstner # synonym
AdEx = aEIF


class AdaptiveReset(object):
    '''
    A two-variable reset:
      V<-Vr
      w<-w+b
    (used in Izhikevich and Brette-Gerstner models)
    '''
    def __init__(self, Vr= -70.6 * mvolt, b=0.0805 * nA):
        self.Vr = Vr
        self.b = b

    def __call__(self, P):
        '''
        Clamps membrane potential at reset value.
        '''
        spikes = P.LS.lastspikes()
        P.vm_[spikes] = self.Vr
        P.w_[spikes] += self.b



speedtrack_desc = 'MultiSpikeGeneratorGroup with a fixed array of spiketimes'

from brian import *
from tracking import *
import gc, time

# All neurons fire simultaneously with the given rate
# The spiketimes are either given as a list of (index, time) pairs or as an 
# array with first column index and second column time  

# Firing rates
rates = [1, 500, 1000, 2000, 5000] #Hz

def run_barrage():
    results = []    
    for r in rates:                    
        print 'Rate: %dHz ' % (r)
        t = TrackSpeed(r)
        results.append((str(r), track(t), r))
        gc.collect()
    return (speedtrack_desc, results)

def plot_results(name, results):
    _, t, arg = zip(*results)
    plot(arg, t, 'o-')
    xlabel('rate (Hz)')
    ylabel('time (s)')
    title(name)


class TrackSpeed(object):
             
    def __init__(self, rate, duration=1): 
        
        N = 100
        dt = defaultclock.dt
        defaultclock.reinit()
        self.duration = duration
        
        #The spike times are shifted by half a dt to center the spikes in the bins
        times = arange(0, duration, 1./rate)             
        spiketimes = [[t*second + dt/2 for t in times] for n in xrange(N)]          
        
        start = time.time()
        G = MultipleSpikeGeneratorGroup(spiketimes)
        self.init_time = time.time() - start
        
        self.net = Network(G)
        self.net.prepare()

    def run(self):
        #simulate the initialization time by waiting...        
        time.sleep(self.init_time)      
                
        self.net.run(self.duration * second)

if __name__ == '__main__':
    name, results = run_barrage()
    print name
    for desc, t, args in results:
        print desc, args, ':', t
    plot_results(name, results)
    show()



# -*- coding:utf-8 -*-
"""
Voltage-dependent STDP from:
Connectivity reflects coding: a model of voltage-based STDP with homeostasis.
Clopath C, Büsing L, Vasilaki E, Gerstner W.
Nat Neurosci. 2010 Mar;13(3):344-52
"""

from brian import *

tau_x = 10 * ms
tau_minus = 10 * ms
tau_plus = 10 * ms
delta_x = 1.
theta = -65 * mV
A_LTD = -0.1
A_LTP = 0.1
delta_u_minus = 5 * mV
delta_u_plus = 5 * mV

eqs_stdp = """
dx/dt=-x/tau_x : 1                      # presynaptic trace
du_minus/dt=(v-u_minus)/tau_minus : volt   # postsynaptic trace 1 
du_plus/dt=(v-u_plus)/tau_plus : volt      # postsynaptic trace 2
v : volt
"""

pre = """
x+=delta_x
w+=clip(u_minus-theta,0,inf)*A_LTD  # A_LTD<0; theta is theta_minus
"""

post = """
u_minus+=delta_u_minus
u_plus+=delta_u_plus
w+=clip(u_plus-theta,0,inf)*x*A_LTP  # A_LTP>0
"""

# The model has hard bounds wmin and wmax
# NB: maybe we should have a rectify function (=clip(x,0,inf))

N = NeuronGroup(2, 'dv/dt=1*Hz:1')
C = Connection(N, N, 'v')
mystdp = STDP(C, eqs=eqs_stdp, pre=pre, post=post, wmax=2 * mV)
mystdp.post_group.v = linked_var(N, 'v')

run(1 * ms)



from brian import *
from scipy import weave
import time
import os

#base, name = os.path.split(__file__)
#print os.path.join(base, 'win32/pthread')
#exit()

N = 10000
x = zeros(N)
nthreads = 1

#if defined(_WIN32) && !defined(__MINGW32__)
  #include "win32/pthread.h"
#else
  #include <pthread.h>
#endif

supp_code = '''
class Work
{
public:
  double *x;
  int istart, iend, N;
  Work(double *x, int istart, int iend, int N) : x(x), istart(istart), iend(iend), N(N) {};
};

void do_work(void* workobj)
{
 Work *w = (Work*)workobj;
 double *x = w->x;
 int istart = w->istart;
 int iend = w->iend;
 int N = w->N;
 for(int i=istart; i<iend; i++)
 {
  x[i] = sin(3.14159*(double)i/(double)N);
 }
}
'''

code = '''
Py_BEGIN_ALLOW_THREADS
Work w = Work(x, 0, N, N);
pthread_t thread;
void *status;
int rc = pthread_create(&thread, NULL, do_work, (void *)&w);
rc = pthread_join(thread, &status);
pthread_exit(NULL);
//do_work((void *)&w);
Py_END_ALLOW_THREADS
'''

start = time.time()
weave.inline(code,
             ['x', 'N', 'nthreads'],
             compiler='mingw32',
             support_code=supp_code,
             headers=['<pthread.h>'],
             extra_compile_args=['-pthread'],
             )
end = time.time()

print end-start

plot(x)
show()



# Iterative voltage-clamp
# R. Brette 2008
from brian import *
#from brian.library.amplifier import *
#from brian.library.electrodes import *

C = 200 * pF
tau = 20 * ms
gl = C / tau
R = 1. / gl
E = 20 * mV # holding potential
g = 150 * nS # initial gain

eqs = '''
dv/dt=(-gl*v+I)/C : volt
I : amp
'''

neuron = NeuronGroup(1, model=eqs)
mon = StateMonitor(neuron, 'v', record=0)

j = 0
current = zeros(500) * amp
@network_operation
def inject():
    global j
    if j < 500:
        neuron.I = current[j]
        j = j + 1
    else:
        neuron.I = 0 * nA

dt = defaultclock.dt
err = 1e15 * mV ** 2
prev_err = err
maxcur = 2 * (C / dt) * E
run(10 * ms)
#E=E*(1-exp(-linspace(0*ms,50*ms,500)/(1*ms)))
for i in range(100):
    j = 0
    run(50 * ms)

    v = mon[0][-500:]
    v += rand(len(v)) * 2 * mV
    err = sum(E - v) # clamp error
    print sqrt(mean((E - v) ** 2)) / float(mV)
    #print err
    #current=g*(E-v)
    #g=1.2*g
    #current+=g*(E-v) # Park's algorithm
    prev = zeros(len(current))
    prev[1:] = (E - v)[0:-1]
    #current+=C/dt*(E-v-exp(-dt/tau)*prev) # Kawato et al
    current += g * (E - v - .99 * prev)
    current = clip(current, -maxcur, maxcur)
    # use best combination of last 2? + new independent component
    #g=2.*g

    run(100 * ms) # rest

plot(mon.times / ms, mon[0] / mV)
show()



"""
Stochastic synaptic transmission.

In this example, two Poisson neurons project to N=20 neurons.
For the first input, the transmission probability increases (0 to 1)
from neuron 0 to neuron 19. 
For the second input, the transmission probability decreases (1 to 0)
from neuron 0 to neuron 19.
The membrane potential of all 20 neurons are shown.
"""
from brian import *

# This is dense, but we could do a sparse one
class ProbabilisticConstructionMatrix(DenseConstructionMatrix):
    def __init__(self,shape,**kwds):
        DenseConstructionMatrix.__init__(self,shape,**kwds)
        self.w0=1. # constant synaptic weight

    def connection_matrix(self):
        return ProbabilisticConnectionMatrix(self)

class ProbabilisticConnectionMatrix(DenseConnectionMatrix):
    def __init__(self, val, **kwds):
        DenseConnectionMatrix.__init__(self, val, **kwds)
        self.w0=val.w0

    def get_rows(self, rows):
        return [(rand(len(self.rows[i]))<self.rows[i])*self.w0 for i in rows]

N=20
tau=5*ms
input=PoissonGroup(2,rates=20*Hz)
neurons=NeuronGroup(N,model='dv/dt=-v/tau : 1')
synapses=Connection(input,neurons,structure=ProbabilisticConstructionMatrix)
synapses.W.w0=0.5 # EPSP size
# Transmission probabilities
synapses.W[0,:]=linspace(0,1,N) # transmission probability between 0 and 1
synapses.W[1,:]=linspace(0,1,N)[::-1] # reverse order for the second input

M=StateMonitor(neurons,'v',record=True)

run(500*ms)

for i in range(N):
    plot(M.times/ms,M[i]+i,'k')
show()



"""
Idea:

Refractoriness should be controlled by allowing user provided code for any of
the following events:

* Start of refractoriness (equivalent to reset in the old scheme)
* During refractoriness (equivalent to refractoriness in the old scheme)
* End of refractoriness (didn't exist in the old scheme)

Code can set any of the following variables:

* is_refractory: directly control whether or not the neuron is in a refractory
  period. If it is set False during refractoriness it will automatically trigger
  the end of refractoriness event.
* refractory_until, refractory_for: control the duration of the refractoriness
  either by setting the end time, or the duration
  
In addition, it is backwards compatible in that you can specify an array of
refractory times, or a state variable which can be used for variable
refractoriness, or of course just a single value.

Users provide code either as Python functions or strings for each of the three
events, but there are some default or built-in types of refractoriness you can
use, notably the reset and clamp. The reset will allow you to just specify a
reset function (like V=Vr) and will set the refractory_for based on either the
constant, array or state variable. The clamp automatically records the
post-reset values of one or more variables and holds them there for the period
of the refractoriness.

Internally, it is implemented using the existing scheme of having an internal
NeuronGroup variable _next_allowed_spiketime which gives the time when the
neuron is next allowed to spike. Each time step, the following sequence of
operations is carried out:

* spikes = threshold()
* Filter spikes based on t>_next_allowed_spiketime
* Trigger during_refractoriness events based on _next_allowed_spiketime>=t
* Trigger end_refractoriness events based on _next_allowed_spiketime<t+dt
* Trigger start_refractoriness events based on filtered spikes

The reason for this sequence of operations is that we do not want to follow
a start_refractoriness with a during_refractoriness, and we want to allow a
during_refractoriness to trigger an end_refractoriness.

In terms of efficiency, checking t>_next_allowed_spiketime is O(num neurons) but
I don't see any way to avoid this if we want to allow variable refractoriness,
and in any case the state update and threshold are already O(num neurons). We
could have optimisations in the case that we know that user code doesn't use the
variable refractoriness features.

TODO:

* Nice syntax for users
* Allow strings for code
  + Python and C versions preferably
"""

from brian import *
import numpy

class TriggeringArray(numpy.ndarray):
    def __new__(subtype, arr, trigger):
        # All numpy.ndarray subclasses need something like this, see
        # http://www.scipy.org/Subclasses
        return numpy.array(arr, copy=False).view(subtype)
    def __init__(self, arr, trigger):
        self.trigger = trigger
        self.triggered = False
    def __getitem__(self, item):
        return asarray(numpy.ndarray.__getitem__(self, item))
    def __setitem__(self, item, value):
        self.trigger(item, value)
        self.triggered = True
        numpy.ndarray.__setitem__(self, item, value)

OrigNeuronGroup = NeuronGroup
class NeuronGroup(OrigNeuronGroup):
    def __init__(self, *args, **kwds):
        refractoriness = kwds.pop('refractoriness')
        OrigNeuronGroup.__init__(self, *args, **kwds)
        self._refractoriness = refractoriness
        self.is_refractory = TriggeringArray(zeros(len(self), dtype=bool),
                                             self._is_refractory_trigger)
        self.refractory_until = TriggeringArray(zeros(len(self)),
                                                self._refractory_until_trigger)
        self.refractory_for = TriggeringArray(zeros(len(self)),
                                              self._refractory_for_trigger)

    def _is_refractory_trigger(self, spikes, values):
        if isinstance(values, numpy.ndarray):
            self._next_allowed_spiketime[spikes[values!=0]] = 1e300 # i.e. refractory for ever
            self._next_allowed_spiketime[spikes[values==0]] = self.clock._t
        else:
            if values:
                self._next_allowed_spiketime[spikes] = 1e300
            else:
                self._next_allowed_spiketime[spikes] = self.clock._t
    
    def _refractory_for_trigger(self, spikes, values):
        self._next_allowed_spiketime[spikes] = self.clock._t+values
    
    def _refractory_until_trigger(self, spikes, values):
        self._next_allowed_spiketime[spikes] = values
        
    def update(self):
        self._state_updater(self) # update the variables
        if self._spiking:
            refrac = self._refractoriness
            spikes = self._threshold(self) # get spikes
            if not isinstance(spikes, numpy.ndarray):
                spikes = array(spikes, dtype=int)
            # Filter refractory spikes
            spikes = spikes[self._next_allowed_spiketime[spikes]<=self.clock._t]
            # Trigger during_refractory based on currently refractory indices
            current_refrac_time = self._next_allowed_spiketime-self.clock._t
            currently_refractory, = (current_refrac_time>=0).nonzero()
            if refrac.during is not None:
                refrac.during(self, currently_refractory)
                # might need to recompute this if is_refractory or others have
                # been used.
                if self.is_refractory.triggered or self.refractory_until.triggered or self.refractory_for.triggered:
                    self.is_refractory.triggered = False
                    self.refractory_until.triggered = False
                    self.refractory_for.triggered = False
                    current_refrac_time[currently_refractory] = self._next_allowed_spiketime[currently_refractory]-self.clock._t
            # Trigger end_refractory based on stop_refractory
            stop_refractory = currently_refractory[current_refrac_time[currently_refractory]<self.clock._dt]
            if refrac.end is not None:
                refrac.end(self, stop_refractory)
            self._next_allowed_spiketime[stop_refractory] = self.clock._t
            # Trigger start_refractory based on spikes array
            if refrac.start is not None:
                refrac.start(self, spikes)
            # Store spikes
            self.LS.push(spikes)

def variable_refractory_start(group, spikes):
    if group._variable_refractory_time:
        if group._refractory_variable is not None:
            refractime = group.state_(group._refractory_variable)
        else:
            refractime = group._refractory_array
        group._next_allowed_spiketime[spikes] = group.clock._t+refractime[spikes]
    else:
        group._next_allowed_spiketime[spikes] = group.clock._t+group._refractory_time

class RefractoryReset(object):
    def __init__(self, reset):
        self.reset = reset
    def __call__(self, group, spikes):
        self.reset(group, spikes)
        variable_refractory_start(group, spikes)

class RefractoryClamp(object):
    def __init__(self, start=None, vars=None):
        self.additional_start = start
        if vars is None:
            vars = (0,)
        self.vars = vars
    def start(self, group, spikes):
        if self.additional_start is not None:
            self.additional_start(group, spikes)
        if not hasattr(self, 'clamped_values'):
            self.clamped_values = dict((var, zeros(len(group))) for var in self.vars)
        for var, vals in self.clamped_values.iteritems():
            vals[spikes] = group.state_(var)[spikes]
    def during(self, group, spikes):
        if not hasattr(self, 'clamped_values'):
            return
        for var, vals in self.clamped_values.iteritems():
            group.state_(var)[spikes] = vals[spikes]

class Refractoriness(object):
    def __init__(self, start=None, during=None, end=None):
        self.start = start
        self.during = during
        self.end = end

if __name__=='__main__':
    eqs = '''
    dV/dt = (2-V)/(10*ms) : 1
    dw/dt = -w/(10*ms) : 1
    drefractory/dt = -refractory/(250*ms) : second
    '''
    if 0:
        def reset(G, spikes):
            G.V[spikes] = 0
        start = RefractoryReset(reset)
        clamp = RefractoryClamp(start, ('V', 'w'))
        start = clamp.start
        during = clamp.during
        def end(G, indices):
            G.w[indices] += 1
    if 1:
        def start(G, spikes):
            G.V[spikes] = 0
            G.w[spikes] += G.refractory[spikes]
            G.is_refractory[spikes] = True
            #G.refractory_for[spikes] = G.refractory[spikes]
            #G.refractory_until[spikes] = 10*ms+G.refractory[spikes]
        def during(G, indices):
            G.V[indices] = 0
            G.is_refractory[indices] = G.w[indices]>1*ms
        def end(G, indices):
            G.w[indices] = 0
    ref = Refractoriness(start, during, end)
    G = NeuronGroup(3, eqs, threshold=1, refractoriness=ref,
                    refractory='refractory',
                    max_refractory=5*ms # dummy value
                    )
    G._variable_refractory_time = True
    G._refractory_variable = 'refractory'
    G._refractory_array = None
    G.refractory = [2*ms, 3*ms, 5*ms]
    M = MultiStateMonitor(G, record=True)
    Msp = SpikeMonitor(G)
    run(40*ms)
    for k in Msp.spikes:
        print k
    for i, var in enumerate(M.vars):
        subplot(len(M.vars), 1, i+1)
        M[var].plot()
        title(var)
    show()
    


import os, re, glob, brian, inspect, compiler, unicodedata, fnmatch
import brian.hears
import itertools
documentable_names = set()
for k, v in itertools.chain(
                brian.__dict__.iteritems(),
                brian.hears.__dict__.iteritems()
                ):
    try:
        if 'brian' in inspect.getsourcefile(v):
            documentable_names.add(k)
    except TypeError:
        pass
os.chdir('../../../examples')

class GlobDirectoryWalker:
    # a forward iterator that traverses a directory tree

    def __init__(self, directory, pattern="*"):
        self.stack = [directory]
        self.pattern = pattern
        self.files = []
        self.index = 0

    def __getitem__(self, index):
        while 1:
            try:
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # pop next directory from stack
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # got a filename
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                if fnmatch.fnmatch(file, self.pattern):
                    return fullname

examplesfnames = [fname for fname in GlobDirectoryWalker('.', '*.py') if 'parallelpythonised' not in fname]
examplespaths = []
examplesbasenames = []
for f in examplesfnames:
    path, file = os.path.split(f)
    path = os.path.normpath(path)
    if path == '.': path = ''
    else: path = path + '_'
    filebase, ext = os.path.splitext(file)
    examplespaths.append(path)
    examplesbasenames.append(filebase)
examplescode = [open(fname, 'r').read() for fname in examplesfnames]
examplesdocs = []
examplesafterdoccode = []
examplesdocumentablenames = []
for code in examplescode:
    codesplit = code.split('\n')
    readingdoc = False
    doc = []
    afterdoccode = ''
    for i in range(len(codesplit)):
        stripped = codesplit[i].strip()
        if stripped[:3] == '"""' or stripped[:3] == "'''":
            if not readingdoc:
                readingdoc = True
            else:
                afterdoccode = '\n'.join(codesplit[i + 1:])
                break
        elif readingdoc:
            doc.append(codesplit[i])
        elif not stripped or stripped[0] == '#':
            pass
        else:
            break
    doc = '\n'.join(doc)
    # next line replaces unicode characters like e-acute with standard ascii representation
    examplesdocs.append(unicodedata.normalize('NFKD', unicode(doc, 'latin-1')).encode('ascii', 'ignore'))
    examplesafterdoccode.append(afterdoccode)
    try:
        examplesdocumentablenames.append(set(compiler.compile(code, '', 'exec').co_names) & documentable_names)
    except SyntaxError:
        print code
        raise
examples = zip(examplesfnames, examplespaths, examplesbasenames, examplescode, examplesdocs, examplesafterdoccode, examplesdocumentablenames)
os.chdir('../docs_sphinx')
for fname, path, basename, code, docs, afterdoccode, documentables in examples:
    title = 'Example: ' + basename
    if len(path): title += ' (' + path[:-1] + ')'
    output = '.. currentmodule:: brian\n\n'
    if len(documentables):
        output += '.. index::\n'
        for dname in documentables:
            output += '   pair: example usage; ' + dname + '\n'
        output += '\n'
    output += '.. _example-' + path + basename + ':\n\n'
    output += title + '\n' + '=' * len(title) + '\n\n'
    output += docs + '\n\n::\n\n'
    output += '\n'.join(['    ' + line for line in afterdoccode.split('\n')])
    output += '\n\n'
    open('examples-' + path + basename + '.txt', 'w').write(output)

mainpage_text = open('examples.txt', 'r').read()
mainpage_text = mainpage_text[:mainpage_text.find('.. EXAMPLES TOCTREE')]
mainpage_text += '.. EXAMPLES TOCTREE\n\n'
mainpage_text += '.. toctree::\n'
mainpage_text += '   :maxdepth: 1\n\n'
curpath = ''
for fname, path, basename, code, docs, afterdoccode, documentables in examples:
    if path != curpath:
        mainpage_text += '\n'
        mainpage_text += '.. _examples-'+path[:-1]+':\n\n'
        mainpage_text += path[:-1] + '\n'
        mainpage_text += '-' * (len(path) - 1) + '\n\n'
        mainpage_text += '.. toctree::\n'
        mainpage_text += '   :maxdepth: 1\n\n'
        curpath = path
    mainpage_text += '   examples-' + path + basename + '\n'
open('examples.txt', 'w').write(mainpage_text)



from brian import *
import pygame
import matplotlib.cm as cm

__all__ = ['RealtimeConnectionMonitor']


class RealtimeConnectionMonitor(NetworkOperation):
    '''
    Realtime monitoring of weight matrix
    
    Short docs:
    
    ``C``
        Connection to monitor
    ``size``
        Dimensions (width, height) of output window, leave as ``None`` to use
        ``C.W.shape``.
    ``scaling``
        If output window dimensions are different to connection matrix, scaling
        is used, options are ``'fast'`` for no interpolation, or ``'smooth'``
        for (slower) smooth interpolation.
    ``wmin, wmax``
        Minimum and maximum weight matrix values, if left to ``None`` then the
        min/max of the weight matrix at each moment is used (and this scaling
        can change over time).
    ``clock``
        Leave to ``None`` for an update every 100ms.
    ``cmap``
        Colour map to use, black and white by default. Get other values from
        ``matplotlib.cm.*``.
    
    Note that this class uses PyGame and due to a limitation with pygame, there
    can be only one window using it. Other options are being considered.
    '''
    def __init__(self, C, size=None, scaling='fast', wmin=None, wmax=None, clock=None, cmap=cm.gray):
        self.C = C
        self.wmin = wmin
        self.wmax = wmax
        self.cmap = cmap
        if clock is None:
            clock = EventClock(dt=100 * ms)
        NetworkOperation.__init__(self, lambda:None, clock=clock)
        pygame.init()
        if size is None:
            width, height = C.W.shape
            self.scaling = None
        else:
            width, height = size
            self.scaling = scaling
        self.width, self.height = width, height
        self.screen = pygame.display.set_mode((width, height))
        self.screen_arr = pygame.surfarray.pixels3d(self.screen)

    def __call__(self):
        W = self.C.W.todense()
        wmin, wmax = self.wmin, self.wmax
        if wmin is None: wmin = amin(W)
        if wmax is None: wmax = amax(W)
        if wmax - wmin < 1e-20: wmax = wmin + 1e-20
        W = self.cmap(clip((W - wmin) / (wmax - wmin), 0, 1), bytes=True)
        if self.scaling is None:
            self.screen_arr[:, :, :] = W[:, :, :3]
        elif self.scaling == 'fast':
            srf = pygame.surfarray.make_surface(W[:, :, :3])
            pygame.transform.scale(srf, (self.width, self.height), self.screen)
        elif self.scaling == 'smooth':
            srf = pygame.surfarray.make_surface(W[:, :, :3])
            pygame.transform.smoothscale(srf, (self.width, self.height), self.screen)
        pygame.display.flip()
        pygame.event.pump()

if __name__ == '__main__':
    Nin = 200
    Nout = 200
    duration = 40 * second

    Fmax = 50 * Hz

    tau = 10 * ms
    taue = 2 * ms
    taui = 5 * ms
    sigma = 0.#0.4
    # Note that factor (tau/taue) makes integral(v(t)) the same when the connection
    # acts on ge as if it acted directly on v.
    eqs = Equations('''
    #dv/dt = -v/tau + sigma*xi/(2*tau)**.5 : 1
    dv/dt = (-v+(tau/taue)*ge-(tau/taui)*gi)/tau + sigma*xi/(2*tau)**.5 : 1
    dge/dt = -ge/taue : 1
    dgi/dt = -gi/taui : 1
    excitatory = ge
    inhibitory = gi
    ''')
    reset = 0
    threshold = 1
    refractory = 0 * ms
    taup = 5 * ms
    taud = 5 * ms
    Ap = .1
    Ad = -Ap * taup / taud * 1.2
    wmax_ff = 0.1
    wmax_rec = wmax_ff
    wmax_inh = wmax_rec

    width = 0.2

    recwidth = 0.2

    Gin = PoissonGroup(Nin)
    Gout = NeuronGroup(Nout, eqs, reset=reset, threshold=threshold, refractory=refractory)
    ff = Connection(Gin, Gout, 'excitatory', structure='dense', weight=rand(Nin, Nout) * wmax_ff)
    for i in xrange(Nin):
        ff[i, :] = (rand(Nout) > .5) * wmax_ff
    rec = Connection(Gout, Gout, 'excitatory')
    for i in xrange(Nout):
        d = abs(float(i) / Nout - linspace(0, 1, Nout))
        d[d > .5] = 1. - d[d > .5]
        dsquared = d ** 2
        prob = exp(-dsquared / (2 * recwidth ** 2))
        prob[i] = -1
        inds = (rand(Nout) < prob).nonzero()[0]
        w = rand(len(inds)) * wmax_rec
        rec[i, inds] = w

    inh = Connection(Gout, Gout, 'inhibitory', sparseness=1, weight=wmax_inh)

    stdp_ff = ExponentialSTDP(ff, taup, taud, Ap, Ad, wmax=wmax_ff)
    stdp_rec = ExponentialSTDP(rec, taup, taud, Ap, Ad, wmax=wmax_rec)

    M = RealtimeConnectionMonitor(ff, size=(500, 500), scaling='smooth',
                                  wmin=0., wmax=wmax_ff,
                                  clock=EventClock(dt=20 * ms),
                                  cmap=cm.jet)

    run(0 * ms)

    @network_operation(clock=EventClock(dt=20 * ms))
    def stimulation():
        Gin.rate = Fmax * exp(-(linspace(0, 1, Nin) - rand())**2 / (2 * width ** 2))

    run(duration, report='stderr')



'''
Inserts licence information in Brian modules.
'''

import os, re

os.chdir('../../../brian') # work from brian/ package directory

exclude = ['insert_licence.py', 'autodiff.py']

license = open('license.txt').read()
# Add comments
pattern = re.compile('^', flags=re.M)
license = pattern.sub('# ', license) + '\n'

def insert_license(filename):
    f_in = open(filename)
    text = f_in.read()
    f_in.close()

    if re.search('Copyright', text) is None:
        f_out = open(filename, 'w')
        f_out.write(license + text)
        f_out.close()
        return True
    else:
        return False

def insert_all(verbose=True):
    # Get all source files
    files = [file for file in os.listdir('') if file[-3:] == '.py' and (file not in exclude)] + \
          ['utils/' + file for file in os.listdir('utils') if file[-3:] == '.py' and (file not in exclude)]

    for file in files:
        if insert_license(file) and verbose:
            print file

if __name__ == '__main__':
    insert_all()



# coding: latin-1
"""
This is a Brian script implementing a benchmark described
in the following review paper:

Simulation of networks of spiking neurons: A review of tools and strategies (2007).
Brette, Rudolph, Carnevale, Hines, Beeman, Bower, Diesmann, Goodman, Harris, Zirpe,
Natschläger, Pecevski, Ermentrout, Djurfeldt, Lansner, Rochel, Vibert, Alvarez, Muller,
Davison, El Boustani and Destexhe.
Journal of Computational Neuroscience 23(3):349-98

Benchmark 2: random network of integrate-and-fire neurons with exponential synaptic currents

Clock-driven implementation with exact subthreshold integration
(but spike times are aligned to the grid)

R. Brette - Oct 2007
--------------------------------------------------------------------------------------
Brian is a simulator for spiking neural networks written in Python, developed by
R. Brette and D. Goodman.
http://brian.di.ens.fr
"""

from brian import *
import time

start_time = time.time()
taum = 20 * ms
taue = 5 * ms
taui = 10 * ms
Vt = -50 * mV
Vr = -60 * mV
El = -49 * mV

eqs = Equations('''
dv/dt  = (ge+gi-(v-El))/taum : volt
dge/dt = -ge/taue : volt
dgi/dt = -gi/taui : volt
''')

P = NeuronGroup(4000, model=eqs, threshold=Vt, reset=Vr, refractory=5 * ms)
P.v = Vr
P.ge = 0 * mV
P.gi = 0 * mV

Pe = P.subgroup(3200)
Pi = P.subgroup(800)
we = (60 * 0.27 / 10) * mV # excitatory synaptic weight (voltage)
wi = (-20 * 4.5 / 10) * mV # inhibitory synaptic weight
Ce = Connection(Pe, P, 'ge', weight=we, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight=wi, sparseness=0.02)
P.v = Vr + rand(len(P)) * (Vt - Vr)

# Record the number of spikes
Me = PopulationSpikeCounter(Pe)
Mi = PopulationSpikeCounter(Pi)
# A population rate monitor
M = PopulationRateMonitor(P)

print "Network construction time:", time.time() - start_time, "seconds"
print len(P), "neurons in the network"
print "Simulation running..."
run(1 * msecond)
start_time = time.time()

run(1 * second)

duration = time.time() - start_time
print "Simulation time:", duration, "seconds"
print Me.nspikes, "excitatory spikes"
print Mi.nspikes, "inhibitory spikes"
plot(M.times / ms, M.smooth_rate(2 * ms, 'gaussian'))
show()
